/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#include "Algos.h"

void
Algos::LinearResample(const vector<SAMPLE> &input, vector<SAMPLE> *output)
{
    double scale = input.size()/(double)output->size();
    
	double index = 0.0;
	for (int i = 0; i < output->size(); i++)
	{
		int indexI = (int)floor(index);
		double t = index - indexI;
		double v0 = input[indexI];
		double v1 = input[indexI + 1];
        
        (*output)[i] = (1.0 - t)*v0 + t*v1;
		index += scale;
	}
    
    // Ensure that the last value is the same in the resampled signal
    (*output)[output->size() - 1] = input[input.size() - 1];
}
