/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef APP_CONTROL_H
#define APP_CONTROL_H

class App;
class AppControl
{
public:
    AppControl(App *app);
    virtual ~AppControl();
    
    virtual const char **GetCommands() const;
    virtual int GetNumCommands() const;

    virtual const char **GetCommandsShort() const;
    virtual int GetNumCommandsShort() const;
    
    virtual void OnSpecialKey(int key, int x, int y);
    virtual void OnSpecialKeyUp(int key, int x, int y);
    virtual void OnKeyUp(unsigned char key, int x, int y);
    virtual void OnKeyboard(unsigned char key, int x, int y);
    virtual void OnMouse(int button, int state, int x, int y);
    virtual void OnMouseDrag(int x, int y);
    virtual void OnMousePassiveMotion(int x, int y);
    virtual void OnMouseWheel(int button, int dir, int x, int y);

    double GetFineDragFactor() const;
    
protected:
    void UpdateMouseCoords(int x, int y);

    void UpdateSourcePicking();

    void CheckMidiNoteKey(unsigned char key, bool keyDown);
    
    App *_app;

    bool _leftMouseDown;
    bool _rightMouseDown;
    bool _middleMouseDown;

    bool _isDraggingBackground;
    bool _ctrlDown;

    double _zoomCenterX;
    double _zoomCenterY;
    bool _altPressed;
    bool _spacePressed;
    bool _shiftPressed;
    
    int _mouseX;
    int _mouseY;
    long _veryHighNumber;
    long _lastDragX;
    long _lastDragY;

    int _lastMouseX;
    int _lastMouseY;
};

#endif
