/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#include "Globals.h"

#include "AudioFile.h"

AudioFile::AudioFile(string name, string path,
                     unsigned int numChan, unsigned long numFrames,
                     double sampleRate, const vector<SAMPLE> &buffer)
{
    _name = name;
    _path = path;
    _nframes = numFrames;
    _nsamples = numFrames * numChan;
    _nchannels = numChan;
    _sampleRate = sampleRate;
    _buffer = buffer;
}

AudioFile::~AudioFile() {}

const string &
AudioFile::GetName() const
{
    return _name;
}

const string &
AudioFile::GetPath() const
{
    return _path;
}

const vector<SAMPLE> *
AudioFile::GetBuffer() const
{
    return &_buffer;
}

unsigned long
AudioFile::GetNumSamples() const
{
    return _nsamples;
}

unsigned long
AudioFile::GetNumFrames() const
{
    return _nframes;
}

unsigned int
AudioFile::GetNumChannels() const
{
    return _nchannels;
}

double
AudioFile::GetSampleRate() const
{
    return _sampleRate;
}
