/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef AUDIO_FILE_H
#define AUDIO_FILE_H

#include <vector>
#include <string>
using namespace std;

#include "Globals.h"

class AudioFile
{
public:
    AudioFile(string name, string path,
              unsigned int numChan, unsigned long numFrames,
              double sampleRate, const vector<SAMPLE> &buffer);
    
    virtual ~AudioFile();

    const string &GetName() const;
    const string &GetPath() const;

    const vector<SAMPLE> *GetBuffer() const;
    
    unsigned long GetNumSamples() const;
    unsigned long GetNumFrames() const;
    unsigned int GetNumChannels() const;
    
    double GetSampleRate() const;
    
protected:
    string _name;
    string _path;
    vector<SAMPLE> _buffer;
    unsigned long _nframes;
    unsigned long _nsamples;
    unsigned int _nchannels;
    double _sampleRate;
};

#endif
