/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "DSP.h"
#include "Parameter.h"
#include "BitCrushEffect.h"

BitCrushEffect::BitCrushEffect()
{
    _ratio = 0.0;

    _params[ON_PARAM] = new ParameterEffectOn(NULL, ON_PARAM);
    _params[RATIO_PARAM] = new ParameterBitCrushEffectRatio(NULL, RATIO_PARAM);
}

BitCrushEffect::~BitCrushEffect()
{
    for (int i = 0; i < NUM_PARAMS; i++)
        delete _params[i];
}

void
BitCrushEffect::Process(vector<SAMPLE> *buffer, int nFrames, int nChan)
{
    if (nChan < 2)
        return;

    UpdateParameters();

    double on = _params[ON_PARAM]->GetValue();
    if (on < 0.5)
        return;

    DSP::BitCrush(buffer, _ratio);
}

void
BitCrushEffect::SetRatio(double ratio)
{
    _ratio = ratio*0.01;
}

Parameter *
BitCrushEffect::GetParameter(ParamIdx paramIdx)
{
    return _params[paramIdx];
}

void
BitCrushEffect::Serialize(Serializer *serializer)
{
    for (int i = 0; i < NUM_PARAMS; i++)
        _params[i]->Serialize(serializer);
}

void
BitCrushEffect::UnSerialize(Serializer *serializer)
{
    for (int i = 0; i < NUM_PARAMS; i++)
        _params[i]->UnSerialize(serializer);

    UpdateParameters();
}

void
BitCrushEffect::UpdateParameters()
{
    double ratio = _params[RATIO_PARAM]->GetValue();
    _ratio = ratio*0.01;
}
