/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#include "Spectrogram.h"
#include "Complex.h"
#include "ChromaScale.h"
#include "SmoothSpace.h"
#include "ChromaOverlapAddProcessor.h"

ChromaOverlapAddProcessor::ChromaOverlapAddProcessor(int fftSize, double sampleRate,
                                                     Spectrogram *spectro,
                                                     int chanNum)
{
    // fftSize/2 ? => magns size is fftSize/2 ...
    _chromaScale = new ChromaScale(fftSize, sampleRate);
    
    _chanNum = chanNum;
    
    _sampleRate = sampleRate;
    
    _spectro = spectro;
    _currentSpectroCol = 0;
}

ChromaOverlapAddProcessor::~ChromaOverlapAddProcessor()
{
    delete _chromaScale;
}

void
ChromaOverlapAddProcessor::ProcessFFT(vector<SAMPLE> *compBuf, int chanNum)
{
    if (chanNum != _chanNum)
        return;

    _spectro->SetCol(_currentSpectroCol, *compBuf);

    // Compute magns
    _tmpBuf.resize(compBuf->size()/4);
    Complex c;
    for (int i = 0; i < compBuf->size()/4; i ++)
    {
        c._re = (*compBuf)[i*2];
        c._im = (*compBuf)[i*2 + 1];
        SAMPLE magn = COMP_MAGN(c);

        _tmpBuf[i] = magn;
    }

    // Compute chrom feature
    _tmpBuf2.resize(compBuf->size()/4);
    MagnsToChroma(_tmpBuf, &_tmpBuf2);
    
    _spectro->SetMagnColDisp(_currentSpectroCol, _tmpBuf2);

    _currentSpectroCol++;
}

void
ChromaOverlapAddProcessor::MagnsToChroma(const vector<SAMPLE> &magns,
                                         vector<SAMPLE> *chroma)
{
    chroma->resize(magns.size());
    memset(chroma->data(), 0, chroma->size()*sizeof(SAMPLE));
    
    // Do not take 0Hz!
    for (int i = 1; i < magns.size(); i++)
    {
        SAMPLE magnVal = magns[i];
        
        int chromaBin = _chromaScale->FreqBinToChromaBin(i*2)*0.5;
        
        if ((chromaBin >= 0) && (chromaBin < chroma->size()))
            (*chroma)[chromaBin] += magnVal;
    }
    
    int divisor = 12;
    if (divisor > chroma->size())
        divisor = chroma->size();
    int winSize = chroma->size()/divisor;

    SmoothSpace smooth;
    smooth.Smooth(chroma, winSize);
}
