/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#include "Scale.h"
#include "ChromaScale.h"

ChromaScale::ChromaScale(int fftSize, double sampleRate)
{
    _c0Freq = ComputeC0Freq();
    
    double toneMult = ComputeToneMult();
    _logToneMultInv = 1.0/log(toneMult);
        
    _fftSize = fftSize;
    _sampleRate = sampleRate;
}

ChromaScale::~ChromaScale() {}

int
ChromaScale::FreqBinToChromaBin(int freqBin)
{
    if (freqBin == 0)
        return 0;

    if (freqBin > _fftSize/2)
        freqBin = _fftSize - freqBin - 1;
    
    double hzPerBin = Scale::HzPerBin(_fftSize, _sampleRate);
    
    double freq = freqBin*hzPerBin;
        
    // See: https://pages.mtu.edu/~suits/NoteFreqCalcs.html
    double fRatio = freq/_c0Freq;
    //double tone = log(fRatio)/log(_toneMult);
    double tone = log(fRatio)*_logToneMultInv;
    
    // Adjust to the axis labels
    tone -= 1.0/12.0;
    
    tone = fmod(tone, 12.0);
    
    double toneNorm = tone/12.0;
    
    double binNumF = toneNorm*_fftSize;
    
    int binNum = round(binNumF);

    return binNum;
}

double
ChromaScale::ComputeC0Freq()
{
    const double ATune = 440.0;
    
    double AMinus1 = ATune/32.0;
    
    double toneMult = pow(2.0, 1.0/12.0);
    
    double ASharpMinus1 = AMinus1*toneMult;
    double BMinus1 = ASharpMinus1*toneMult;
    
    double C0 = BMinus1*toneMult;
    
    return C0;
}

double
ChromaScale::ComputeToneMult()
{
    double toneMult = pow(2.0, 1.0/12.0);

    return toneMult;
}
