/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Spectrogram.h"
#include "ChromaScale.h"
#include "ChromaToSamplesOverlapAddProcessor.h"

ChromaToSamplesOverlapAddProcessor::
ChromaToSamplesOverlapAddProcessor(int fftSize, double sampleRate,
                                   Spectrogram *spectro)
{
    _fftSize = fftSize;
    _sampleRate = sampleRate;
    
    _chromaScale = new ChromaScale(fftSize, sampleRate);
    
    _spectro = spectro;
    _currentSpectroColNum = 0.0;

    _selChroma[0] = 0.0;
    _selChroma[1] = 1.0;

    _timeStretch = 1.0;
}

ChromaToSamplesOverlapAddProcessor::~ChromaToSamplesOverlapAddProcessor()
{
    delete _chromaScale;
}

OverlapAddProcessor *
ChromaToSamplesOverlapAddProcessor::Clone()
{
    ChromaToSamplesOverlapAddProcessor *clone =
        new ChromaToSamplesOverlapAddProcessor(_fftSize, _sampleRate, _spectro);
    clone->_currentSpectroColNum = _currentSpectroColNum;
    for (int i = 0; i < 2; i++)
        clone->_selChroma[i] = _selChroma[i];
    clone->_timeStretch = _timeStretch;
    
    return clone;
}

void
ChromaToSamplesOverlapAddProcessor::SetSpectroColNum(double colNum)
{
    _currentSpectroColNum = colNum;
}

void
ChromaToSamplesOverlapAddProcessor::SetSelFreq(double selFreq[2])
{
    _selChroma[0] = selFreq[0];
    _selChroma[1] = selFreq[1];
}

void
ChromaToSamplesOverlapAddProcessor::SetTimeStretch(double timeStretch)
{
    _timeStretch = timeStretch;
}

void
ChromaToSamplesOverlapAddProcessor::ProcessFFT(vector<SAMPLE> *compBuf, int chanNum)
{
    if (_currentSpectroColNum >= _spectro->GetNumCols())
        return;
        
    vector<SAMPLE> spectroCompBuf;
    _spectro->GetCol(_currentSpectroColNum, &spectroCompBuf);

    int selChroma[2] =
        { (int)(_selChroma[0]*spectroCompBuf.size()*0.5),
          (int)(_selChroma[1]*spectroCompBuf.size()*0.5) };
    for (int i = 0; i < spectroCompBuf.size(); i += 2)
    {
        int chromaBin = _chromaScale->FreqBinToChromaBin(i*0.5);
            
        if ((chromaBin < selChroma[0]) || (chromaBin > selChroma[1]))
        {
            spectroCompBuf[i] = 0.0;
            spectroCompBuf[i + 1] = 0.0;
        }
    }
    
    memcpy(compBuf->data(), spectroCompBuf.data(), spectroCompBuf.size()*sizeof(SAMPLE));

    _currentSpectroColNum += _timeStretch;
}
