/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef COMPLEX_H
#define COMPLEX_H

#include <math.h>

#include "Globals.h"

typedef struct
{
    SAMPLE _re;
    SAMPLE _im;
} Complex;
    
#define COMP_MAGN(__x__) (sqrt((__x__)._re * (__x__)._re + (__x__)._im * (__x__)._im))
#define COMP_PHASE(__x__) (atan2((__x__)._im, (__x__)._re))

#define MAGN_PHASE_COMP(__MAGN__, __PHASE__, __COMP__)  \
__COMP__._re = __MAGN__*cos(__PHASE__);                 \
__COMP__._im = __MAGN__*sin(__PHASE__);

#define COMP_ADD(__COMP0__, __COMP1__, __RESULT__)      \
__RESULT__._re = __COMP0__._re + __COMP1__._re;         \
__RESULT__._im = __COMP0__._im + __COMP1__._im;

#define COMP_MULT(__COMP0__, __COMP1__, __RESULT__)                          \
__RESULT__._re = __COMP0__._re*__COMP1__._re - __COMP0__._im*__COMP1__._im;  \
__RESULT__._im = __COMP0__._im*__COMP1__._re + __COMP0__._re*__COMP1__._im;

// See: https://courses.lumenlearning.com/ivytech-collegealgebra/chapter/multiply-and-divide-complex-numbers/
#define COMP_DIV(__COMP0__, __COMP1__, __RESULT__)                                       \
{ BL_FLOAT denom = __COMP1__._re*__COMP1__._re + __COMP1__._im*__COMP1__._im;            \
BL_FLOAT denomInv = 1.0/denom;                                                           \
__RESULT__._re = (__COMP0__._re*__COMP1__._re + __COMP0__._im*__COMP1__._im)*denomInv;   \
__RESULT__._im = (__COMP1__._re*__COMP0__._im - __COMP0__._re*__COMP1__._im)*denomInv; }

#endif
