/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "DSP.h"
#include "Convolver.h"

Convolver::Convolver(const vector<SAMPLE> &ir)
{
    _ir = ir;
}

Convolver::~Convolver() {}

void
Convolver::Reset()
{
    _prevAdditionalSamples.clear();
}

void
Convolver::SetIR(const vector<SAMPLE> &ir)
{
    _ir = ir;
}

void
Convolver::Process(const vector<SAMPLE> &inSamples,
                   vector<SAMPLE> *outSamples)
{
    int xs = inSamples.size();
    int hs = _ir.size();
    int cs = xs + hs - 1;
    
    outSamples->resize(cs);
    DSP::Silent(outSamples);
    
    for(int i = 0; i < cs; i++)
    {
        (*outSamples)[i] = 0.0;

        unsigned long jmin = (i >= xs - 1) ? i - xs + 1 : 0;
        unsigned long jmax = (i < hs - 1) ? i : hs - 1;
        
        for(int j = jmin; j <= jmax; j++)
            (*outSamples)[i] += inSamples[i - j] * _ir[j];
    }

    // Add the previous remaining samples
    for (int i = 0; i < _prevAdditionalSamples.size(); i++)
    {
        if (i < outSamples->size())
            (*outSamples)[i] += _prevAdditionalSamples[i];
    }

    // Keep the remaining samples for next time
    _prevAdditionalSamples.resize(outSamples->size() - inSamples.size());
    
    memcpy(_prevAdditionalSamples.data(),
           &outSamples->data()[inSamples.size()],
           _prevAdditionalSamples.size()*sizeof(SAMPLE));
                               
    // Crop end
    outSamples->resize(inSamples.size());
}
