/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef DSP_H
#define DSP_H

#include <vector>
using namespace std;

#include "Globals.h"

class DSP
{
public:
    static void Normalize(vector<SAMPLE> *buffer);
    static void AutomaticClipping(vector<SAMPLE> *buffer);

    static void Interleave(vector<SAMPLE> *buffer, const vector<SAMPLE> &lBuf,
                           const vector<SAMPLE> &rBuf);
    static void DeInterleave(const vector<SAMPLE> &buffer,
                             vector<SAMPLE> *lBuf, vector<SAMPLE> *rBuf);

    static SAMPLE CmpChannels(const vector<SAMPLE> &buffer);
    
    static void FadeIn(vector<SAMPLE> *buffer, int nChan, int numSamplesFade);
    static void FadeOut(vector<SAMPLE> *buffer, int nChan, int numSamplesFade);

    static void Balance(vector<SAMPLE> *buffer, int nChan, double balance);

    static void BitCrush(vector<SAMPLE> *buffer, double ratio);
    
    static void Saturation(vector<SAMPLE> *buffer, double ratio);

    static void Silent(vector<SAMPLE> *buffer);
    static bool IsSilence(const vector<SAMPLE> &buffer);
    
    static void Reverse(vector<SAMPLE> *buffer, int nChan);

    static void Clip(SAMPLE *s, SAMPLE minS, SAMPLE maxS);
};

#endif
