/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef DELAY_EFFECT_H
#define DELAY_EFFECT_H

#include "Effect.h"

class Parameter;
class DelayLine;
class DelayEffect : public Effect
{
public:
    enum ParamIdx
    {
        ON_PARAM = 0,
        TIME_PARAM,
        FEEDBACK_PARAM,
        WET_MIX_PARAM,
        NUM_PARAMS
    };
    
    DelayEffect();
    virtual ~DelayEffect();
    
    void Process(vector<SAMPLE> *buffer, int nFrames, int nChan) override;

    void SetTime(double timeMs);
    void SetFeedback(double feedback);
    void SetWetMix(double wetMix);

    Parameter *GetParameter(ParamIdx paramIdx);

    void Serialize(Serializer *serializer) override;
    void UnSerialize(Serializer *serializer) override;
    
protected:
    void UpdateParameters();
        
    int _timeSamples;
    
    double _feedback;
    double _wetMix;

    Parameter *_params[NUM_PARAMS];

    DelayLine *_delayLines[CHANNELS];
};

#endif
