/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Effect.h"
#include "VolumeEffect.h"
#include "StereoEffect.h"
#include "BitCrushEffect.h"
#include "DelayEffect.h"
#include "ReverbEffect.h"
#include "UIVolumeEffect.h"
#include "UIStereoEffect.h"
#include "UIBitCrushEffect.h"
#include "UISaturationEffect.h"
#include "UIDelayEffect.h"
#include "UIReverbEffect.h"
#include "UIEffectsCanvas.h"
#include "EffectsBox.h"

EffectsBox::EffectsBox()
{
    // Volume effect
    VolumeEffect *volumeEffect = new VolumeEffect();
    _effects[EffectsBox::EFFECT_VOLUME] = volumeEffect;
    
    // Stereo effect
    StereoEffect *stereoEffect = new StereoEffect();
    _effects[EffectsBox::EFFECT_STEREO] = stereoEffect;

    // Bit crush effect
    BitCrushEffect *bitCrushEffect = new BitCrushEffect();
    _effects[EffectsBox::EFFECT_BIT_CRUSH] = bitCrushEffect;

    // Saturation effect
    SaturationEffect *saturationEffect = new SaturationEffect();
    _effects[EffectsBox::EFFECT_SATURATION] = saturationEffect;
    
    // Delay effect
    DelayEffect *delayEffect = new DelayEffect();
    _effects[EffectsBox::EFFECT_DELAY] = delayEffect;
    
    // Reverb effect
    ReverbEffect *reverbEffect = new ReverbEffect();
    _effects[EffectsBox::EFFECT_REVERB] = reverbEffect;
}

EffectsBox::~EffectsBox()
{
    for (int i = 0; i < NUM_EFFECTS; i++)
        delete _effects[i];
}

void
EffectsBox::RegisterUI(UIEffectsCanvas *ui)
{
    // Volume effect
    VolumeEffect *volumeEffect = (VolumeEffect *)_effects[EffectsBox::EFFECT_VOLUME];
        
    UIVolumeEffect *volumeEffectUI =
        (UIVolumeEffect *)ui->GetEffect(EffectsBox::EFFECT_VOLUME);
    Parameter *onParamVolume = volumeEffect->GetParameter(VolumeEffect::ON_PARAM);
    volumeEffectUI->SetParameter(VolumeEffect::ON_PARAM, onParamVolume);
    Parameter *volumeParam = volumeEffect->GetParameter(VolumeEffect::VOLUME_PARAM);
    volumeEffectUI->SetParameter(VolumeEffect::VOLUME_PARAM, volumeParam);
    Parameter *muteParam = volumeEffect->GetParameter(VolumeEffect::MUTE_PARAM);
    volumeEffectUI->SetParameter(VolumeEffect::MUTE_PARAM, muteParam);
    
    // Stereo effect
    StereoEffect *stereoEffect = (StereoEffect *)_effects[EffectsBox::EFFECT_STEREO];
        
    UIStereoEffect *stereoEffectUI =
        (UIStereoEffect *)ui->GetEffect(EffectsBox::EFFECT_STEREO);
    Parameter *onParamStereo = stereoEffect->GetParameter(StereoEffect::ON_PARAM);
    stereoEffectUI->SetParameter(StereoEffect::ON_PARAM, onParamStereo);
    Parameter *panParam = stereoEffect->GetParameter(StereoEffect::PAN_PARAM);
    stereoEffectUI->SetParameter(StereoEffect::PAN_PARAM, panParam);
    Parameter *widthParam = stereoEffect->GetParameter(StereoEffect::WIDTH_PARAM);
    stereoEffectUI->SetParameter(StereoEffect::WIDTH_PARAM, widthParam);
    
    // Bit crush effect
    BitCrushEffect *bitCrushEffect =
        (BitCrushEffect *)_effects[EffectsBox::EFFECT_BIT_CRUSH];
        
    UIBitCrushEffect *bitCrushEffectUI =
        (UIBitCrushEffect *)ui->GetEffect(EffectsBox::EFFECT_BIT_CRUSH);
    Parameter *onParamBitCrush = bitCrushEffect->GetParameter(BitCrushEffect::ON_PARAM);
    bitCrushEffectUI->SetParameter(BitCrushEffect::ON_PARAM, onParamBitCrush);
    Parameter *ratioParam = bitCrushEffect->GetParameter(BitCrushEffect::RATIO_PARAM);
    bitCrushEffectUI->SetParameter(BitCrushEffect::RATIO_PARAM, ratioParam);

    // Saturation effect
    SaturationEffect *saturationEffect =
        (SaturationEffect *)_effects[EffectsBox::EFFECT_SATURATION];
        
    UISaturationEffect *saturationEffectUI =
        (UISaturationEffect *)ui->GetEffect(EffectsBox::EFFECT_SATURATION);
    Parameter *onParamSaturation =
        saturationEffect->GetParameter(SaturationEffect::ON_PARAM);
    saturationEffectUI->SetParameter(SaturationEffect::ON_PARAM, onParamSaturation);
    Parameter *ratioParamSat =
        saturationEffect->GetParameter(SaturationEffect::RATIO_PARAM);
    saturationEffectUI->SetParameter(SaturationEffect::RATIO_PARAM, ratioParamSat);
    
    // Delay effect
    DelayEffect *delayEffect = (DelayEffect *)_effects[EffectsBox::EFFECT_DELAY];
        
    UIDelayEffect *delayEffectUI =
        (UIDelayEffect *)ui->GetEffect(EffectsBox::EFFECT_DELAY);
    Parameter *onParamDelay = delayEffect->GetParameter(DelayEffect::ON_PARAM);
    delayEffectUI->SetParameter(DelayEffect::ON_PARAM, onParamDelay);
    Parameter *timeParam = delayEffect->GetParameter(DelayEffect::TIME_PARAM);
    delayEffectUI->SetParameter(DelayEffect::TIME_PARAM, timeParam);
    Parameter *feedbackParam = delayEffect->GetParameter(DelayEffect::FEEDBACK_PARAM);
    delayEffectUI->SetParameter(DelayEffect::FEEDBACK_PARAM, feedbackParam);
    Parameter *wetMixParam = delayEffect->GetParameter(DelayEffect::WET_MIX_PARAM);
    delayEffectUI->SetParameter(DelayEffect::WET_MIX_PARAM, wetMixParam);
    
    // Reverb effect
    ReverbEffect *reverbEffect = (ReverbEffect *)_effects[EffectsBox::EFFECT_REVERB];
        
    UIReverbEffect *reverbEffectUI =
        (UIReverbEffect *)ui->GetEffect(EffectsBox::EFFECT_REVERB);
    Parameter *onParamReverb = reverbEffect->GetParameter(ReverbEffect::ON_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::ON_PARAM, onParamReverb);
    Parameter *orderParam = reverbEffect->GetParameter(ReverbEffect::ORDER_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::ORDER_PARAM, orderParam);
    Parameter *feedDryInParam = reverbEffect->GetParameter(ReverbEffect::FEED_DRY_IN_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::FEED_DRY_IN_PARAM, feedDryInParam);
    Parameter *feedWetOutParam = reverbEffect->
        GetParameter(ReverbEffect::FEED_WET_OUT_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::FEED_WET_OUT_PARAM, feedWetOutParam);
    Parameter *dryMixParam = reverbEffect->GetParameter(ReverbEffect::DRY_MIX_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::DRY_MIX_PARAM, dryMixParam);
    Parameter *delayTimeParam = reverbEffect->GetParameter(ReverbEffect::DELAY_TIME_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::DELAY_TIME_PARAM, delayTimeParam);
    Parameter *decayParam = reverbEffect->GetParameter(ReverbEffect::DECAY_PARAM);
    reverbEffectUI->SetParameter(ReverbEffect::DECAY_PARAM, decayParam);
}

void
EffectsBox::Process(vector<SAMPLE> *buffer, int nFrames, int nChan)
{
    for (int i = 0; i < NUM_EFFECTS; i++)
        _effects[i]->Process(buffer, nFrames, nChan);
}

void
EffectsBox::Serialize(Serializer *serializer)
{
    for (int i = 0; i < NUM_EFFECTS; i++)
        _effects[i]->Serialize(serializer);
}

void
EffectsBox::UnSerialize(Serializer *serializer)
{
    for (int i = 0; i < NUM_EFFECTS; i++)
        _effects[i]->UnSerialize(serializer);
}
