/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef ENVELOPE_H
#define ENVELOPE_H

#include <cmath>
#include <math.h>

#include <iostream>

#include "Globals.h"

#include "Stk.h"

#include <vector>
using namespace std;

class Serializer;
class Envelope
{
public:
    enum
    {
      HANNING = 0,
      TRIANGLE,
      EXPDEC,
      REXPDEC,
      SINC,
      HAND_DRAWN,
      NUM_ENVELOPES
    };

    Envelope(unsigned long length = ENVELOPE_LEN);
    virtual ~Envelope();
    
    const vector<SAMPLE> &GetEnvelope(int envelopeType);
    void SetEnvelope(int envelopeType, const vector<SAMPLE> &data);
    int GetEnvelopeSize();
    
    int GetNumEnvelopes();

    virtual void Serialize(Serializer *serializer);
    virtual void UnSerialize(Serializer *serializer);
    
protected:
    // Generate envelopes
    void GenerateEnvelopes();
    
    void Hanning(vector<SAMPLE> *envelope);
    void Triangle(vector<SAMPLE> *envelope);
    void Expdec(vector<SAMPLE> *forEnv, vector<SAMPLE> *revEnv);
    void Sinc(vector<SAMPLE> *envelope, int numZeroCross = 6);

    int _envSize;
    
    vector<SAMPLE> _hanningEnv;
    vector<SAMPLE> _triEnv;
    vector<SAMPLE> _expDecEnv;
    vector<SAMPLE> _rexpDecEnv;
    vector<SAMPLE> _sincEnv;
    vector<SAMPLE> _handDrawnEnv;
};

#endif
