/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <math.h>

#include "KeyMap.h"

KeyMap::KeyMap()
{
    for (int i = 0; i < NUM_KEYS; i++)
        _keyMap[i] = false;
}

KeyMap::~KeyMap() {}

void
KeyMap::SetKeyValue(int keyNum, bool value)
{
    if ((keyNum < 0) || (keyNum > 10))
        return;

    _keyMap[keyNum] = value;
}

void
KeyMap::GetPitchMap(double pitchMap[NUM_KEYS]) const
{
    double factor = pow(2.0, 1.0/12.0);

    double pitch = 1.0;

    for (int i = 0; i < NUM_KEYS; i++)
    {
        // Set to undefined pitch
        pitchMap[i] = -1.0;
        
        if (_keyMap[i])
            pitchMap[i] = pitch;

        pitch *= factor;
    }

    int numKeysPressed = 0;
    for (int i = 0; i < NUM_KEYS; i++)
    {
        if (_keyMap[i])
            numKeysPressed++;
    }

    // If no key pressed, play only the fundamental
    if (numKeysPressed == 0)
        pitchMap[0] = 1.0;
}

void
KeyMap::GetKeyMap(bool keyPressed[NUM_KEYS]) const
{
    for (int i = 0; i < NUM_KEYS; i++)
        keyPressed[i] = _keyMap[i];
}

bool
KeyMap::AnyKeyPressed() const
{
    for (int i = 0; i < NUM_KEYS; i++)
    {
        if (_keyMap[i])
            return true;
    }

    return false;
}
