/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "MidiQueue.h"    

// MidiMessage
MidiQueue::MidiMessage::MidiMessage() {}

MidiQueue::MidiMessage::MidiMessage(double deltaTime, const vector<unsigned char> &message)
{
    _deltaTime = deltaTime;
    _message = message;
}

MidiQueue::MidiMessage::~MidiMessage() {}

double
MidiQueue::MidiMessage::GetTime() const
{
    return _deltaTime;
}

int
MidiQueue::MidiMessage::GetChannel() const
{
    if ((_message[0] & 0xF0) != 0xF0)
        return (_message[0] & 0xF) + 1;
    return 0;
}

bool
MidiQueue::MidiMessage::IsNoteOn() const
{
    unsigned int e = (_message[0] >> 4);
    if (e == 0x09)
        return true;

    return false;
}

bool
MidiQueue::MidiMessage::IsNoteOff() const
{
    unsigned int e = (_message[0] >> 4);
    if (e == 0x08)
        return true;

    return false;
}

int
MidiQueue::MidiMessage::GetNoteNumber() const
{
    return _message[1];
}
bool
MidiQueue::MidiMessage::IsPitchWheel() const
{
    unsigned int e = (_message[0] >> 4);
    if (e == 0x0e)
        return true;

    return false;
}
double
MidiQueue::MidiMessage::GetPitchWheel() const
{
    int val = (_message[2] << 7) + _message[1];
    return (double)(val - 8192)/ 8192.0;
}

// MidiQueue
MidiQueue::MidiQueue() {}

MidiQueue::~MidiQueue() {}

void
MidiQueue::PushMessage(double deltaTime, const vector<unsigned char> &message)
{
    MidiMessage msg(deltaTime, message);
    _messages.push_back(msg);
}

int
MidiQueue::GetSize() const
{
    return _messages.size();
}

bool
MidiQueue::PopMessage(MidiMessage *msg)
{
    if (_messages.empty())
        return false;

    *msg = _messages[0];
    _messages.pop_front();
    
    return true;
}
