/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef MIDI_QUEUE_H
#define MIDI_QUEUE_H

#include <vector>
#include <deque>
using namespace std;

class MidiQueue
{
public:
    class MidiMessage
    {
    public:
        MidiMessage();
        MidiMessage(double deltaTime, const vector<unsigned char> &message);
        virtual ~MidiMessage();

        double GetTime() const;

        int GetChannel() const;
        
        bool IsNoteOn() const;
        bool IsNoteOff() const;
        int GetNoteNumber() const;

        bool IsPitchWheel() const;
        double GetPitchWheel() const;
        
    protected:
        double _deltaTime;
        vector<unsigned char> _message;
    };
    
    MidiQueue();
    virtual ~MidiQueue();

    void PushMessage(double deltaTime, const vector<unsigned char> &message);

    int GetSize() const;
    bool PopMessage(MidiMessage *msg);

protected:
    deque<MidiMessage> _messages;
};

#endif
