/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Globals.h"
#include "MyRtAudio.h"

using namespace std;

MyRtAudio::~MyRtAudio()
{
    delete _audio;
}

MyRtAudio::MyRtAudio(unsigned int numIns,
                     unsigned int numOuts,
                     unsigned int srate,
                     unsigned int *bufferSize,
                     RtAudioFormat format,
                     bool showWarnings)
{
    _audio = NULL;
 
    try
    {
        _audio = new RtAudio();
    }
    catch(RtAudioError &err)
    {
        err.printMessage();
        exit(1);
    }
    
    _numInputs = numIns;
    _numOutputs = numOuts;
    
    if (_audio->getDeviceCount() < 1)
    {
        cout << "No Audio Devices Found!" << endl;
        exit(1);
    }
    
    _audio->showWarnings(showWarnings);
    
    _bufferSize = bufferSize;
    
    _sampleRate = srate;
    
    _format = format;
}


void
MyRtAudio::OpenStream(RtAudioCallback callback)
{
    RtAudio::StreamOptions options;
    // options.flags |= RTAUDIO_NONINTERLEAVED;
    
    RtAudio::StreamParameters iParams;
    RtAudio::StreamParameters oParams;
    
    // I/O params
    iParams.deviceId = _audio->getDefaultInputDevice();
    iParams.nChannels = _numInputs;
    iParams.firstChannel = 0;
    oParams.deviceId = _audio->getDefaultOutputDevice();
    oParams.nChannels = _numOutputs;
    oParams.firstChannel = 0;
    
    // Open stream
    _audio->openStream(&oParams, &iParams,
                       FORMAT, _sampleRate,
                       _bufferSize,
                       callback, NULL, &options); 
}

unsigned int
MyRtAudio::GetBufferSize()
{        
    cerr << "Buffer size defined by RtAudio: " <<
        (*_bufferSize) << endl;

    return (*_bufferSize);
}           

void
MyRtAudio::StartStream()
{
    _audio->startStream();
}

void
MyRtAudio::StopStream()
{
    try
    {
        _audio->stopStream();
    }
    catch(RtAudioError &err)
    {
        err.printMessage();
    }
}

void
MyRtAudio::CloseStream()
{
    _audio->closeStream();
}

void
MyRtAudio::ReportStreamLatency()
{
    cout << "Stream Latency: " << _audio->getStreamLatency() <<
        " frames" << endl;    
}
