/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <vector>
using namespace std;

#include "RtMidi.h"
#include "MyRtMidiOut.h"

MyRtMidiOut::MyRtMidiOut()
{    
    try
    {
        _midiOut = new RtMidiOut();
    }
    catch (RtMidiError &error)
    {
        error.printMessage();
    }
}

MyRtMidiOut::~MyRtMidiOut()
{
    if (_midiOut != NULL)
        delete _midiOut;   
}
    
int
MyRtMidiOut::Open(int midiOutPort)
{
    try
    {
        _midiOut->openPort(midiOutPort);
    }
    catch (RtMidiError &error)
    {
        error.printMessage();
    }
    
    // Port should be portCount - 1 ?
    int port = _midiOut->getPortCount();
    if (port >= 0)
        return port;
    
    return -1;
}

void
MyRtMidiOut::SendNoteOn(int noteNum)
{
    int channel = 0;
    
    vector<unsigned char> message;
    message.resize(3);
    message[0] = 0x90 | (unsigned char)channel;
    message[1] = noteNum;
    message[2] = 127; // velocity
    
    _midiOut->sendMessage(&message);
}

void
MyRtMidiOut::SendNoteOff(int noteNum)
{
    int channel = 0;
    
    vector<unsigned char> message;
    message.resize(3);
    message[0] = 0x80 | (unsigned char)channel;
    message[1] = noteNum;
    message[2] = 127; // velocity
        
    _midiOut->sendMessage(&message);
}
