/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef OVERLAP_ADD_H
#define OVERLAP_ADD_H

#include "Globals.h"

#define kiss_fft_scalar SAMPLE
#include "kiss_fft.h"

#include "CircularBuffer.h"

class OverlapAddProcessor
{
public:
    OverlapAddProcessor();
    virtual ~OverlapAddProcessor();

    virtual OverlapAddProcessor *Clone() { return NULL; };
    
    virtual void ProcessFFT(vector<SAMPLE> *compBuf, int chanNum);
    virtual void ProcessOutSamples(vector<SAMPLE> *buff, int chanNum);

    virtual void SetSpectroColNum(double colNum) {};
    virtual void SetSelFreq(double selFreq[2]) {};
    virtual void SetTimeStretch(double timeStretch) {};
};

class OverlapAdd
{
public:
    OverlapAdd(int fftSize, int overlap, bool fft, bool ifft);
    virtual ~OverlapAdd();

    void SetSynthTimeStretch(double timeStretch);
    
    void AddProcessor(OverlapAddProcessor *processor);
    
    void Feed(const vector<SAMPLE> &samples, int blockSize, int nChan);

    void GetOutSamples(vector<SAMPLE> *samples);
    void ClearOutSamples();
    void FlushOutSamples(int numToFlush);
    
protected:
    virtual void ProcessFFT(vector<SAMPLE> *compBuf, int chanNum);
    virtual void ProcessOutSamples(vector<SAMPLE> *buff, int chanNum);

    // win must have been resized before
    virtual void MakeWindow(vector<SAMPLE> *win);

    vector<OverlapAddProcessor *> _processors;
    
    int _fftSize;
    int _overlap;

    bool _fftFlag;
    bool _ifftFlag;

    double _synthTimeStretch;
    
    CircularBuffer<SAMPLE> _circSampBufsIn[2];
    CircularBuffer<SAMPLE> _circSampBufsOut[2];
    
    vector<SAMPLE> _tmpSampBufIn;
    vector<SAMPLE> _tmpSampBufOut;
    vector<SAMPLE> _tmpCompBufIn;
    vector<SAMPLE> _tmpCompBufOut;

    vector<SAMPLE> _tmpSynthZeroBuf;
    
    vector<SAMPLE> _anaWin;
    vector<SAMPLE> _synthWin;

    kiss_fft_cfg _forwardFFT;
    kiss_fft_cfg _backwardFFT;

    vector<SAMPLE> _outSamples;
};

#endif
