/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef PARAMETER_H
#define PARAMETER_H

class ParameterProvider;
class App;
class UIParameter;
class Serializer;
class Parameter
{
public:
    Parameter(App *app, int paramIdx);
    virtual ~Parameter();

    virtual void SetValue(ParameterProvider *provider, double value) = 0;
    virtual void SetValue(double value) = 0;
    virtual void UpdateValue() = 0;
    double GetValue() const;
    double GetDefaultValue() const;
    const char *GetValueStr() const;
    
    void SetNormalizedValue(double value);
    double GetNormalizedValue() const;
    
    void SetSensivity(double sensivity);
    double GetSensivity() const;
    
    int GetIndex() const;
    
    UIParameter *GetUI();

    void Serialize(Serializer *serializer);
    void UnSerialize(Serializer *serializer);
    
protected:
    void SetName(const char *name);
    void SetValueStr(const char *valueStr);

    void CheckValueBounds(double *value);
    void SetValueCheckBounds(double value);
    
    App *_app;
    int _idx;
    
    double _value;
    double _minValue;
    double _maxValue;
    double _defaultValue;

    // For non linear parameters
    double _powFactor;
    
    UIParameter *_uiParam;

    double _sensivity;

    char _valueStr[255];
};

class ParameterNumGrains : public Parameter
{
public:
    ParameterNumGrains(App *app, int paramId);
    virtual ~ParameterNumGrains();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterDuration : public Parameter
{
public:
    ParameterDuration(App *app, int paramId);
    virtual ~ParameterDuration();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterEnvelope : public Parameter
{
public:
    ParameterEnvelope(App *app, int paramId);
    virtual ~ParameterEnvelope();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterMotionX : public Parameter
{
public:
    ParameterMotionX(App *app, int paramId);
    virtual ~ParameterMotionX();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterMotionY : public Parameter
{
public:
    ParameterMotionY(App *app, int paramId);
    virtual ~ParameterMotionY();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterMotionXY : public Parameter
{
public:
    ParameterMotionXY(App *app, int paramId);
    virtual ~ParameterMotionXY();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterDirection : public Parameter
{
public:
    ParameterDirection(App *app, int paramId);
    virtual ~ParameterDirection();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterOverlap : public Parameter
{
public:
    ParameterOverlap(App *app, int paramId);
    virtual ~ParameterOverlap();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterPitch : public Parameter
{
public:
    ParameterPitch(App *app, int paramId);
    virtual ~ParameterPitch();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterLFOFreq : public Parameter
{
public:
    ParameterLFOFreq(App *app, int paramId);
    virtual ~ParameterLFOFreq();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterLFOAmp : public Parameter
{
public:
    ParameterLFOAmp(App *app, int paramId);
    virtual ~ParameterLFOAmp();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterSpatialize : public Parameter
{
public:
    ParameterSpatialize(App *app, int paramId);
    virtual ~ParameterSpatialize();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterVolume : public Parameter
{
public:
    ParameterVolume(App *app, int paramId);
    virtual ~ParameterVolume();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterRange : public Parameter
{
public:
    ParameterRange(App *app, int paramId);
    virtual ~ParameterRange();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterSoundRectType : public Parameter
{
public:
    ParameterSoundRectType(App *app, int paramId);
    virtual ~ParameterSoundRectType();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterColorRange : public Parameter
{
public:
    ParameterColorRange(App *app, int paramId);
    virtual ~ParameterColorRange();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterColorContrast : public Parameter
{
public:
    ParameterColorContrast(App *app, int paramId);
    virtual ~ParameterColorContrast();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterSoundRectOrientation : public Parameter
{
public:
    ParameterSoundRectOrientation(App *app, int paramId);
    virtual ~ParameterSoundRectOrientation();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterTimeStretch : public Parameter
{
public:
    ParameterTimeStretch(App *app, int paramId);
    virtual ~ParameterTimeStretch();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterStereoEffectPan : public Parameter
{
public:
    ParameterStereoEffectPan(App *app, int paramId);
    virtual ~ParameterStereoEffectPan();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterStereoEffectWidth : public Parameter
{
public:
    ParameterStereoEffectWidth(App *app, int paramId);
    virtual ~ParameterStereoEffectWidth();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterReverbEffectOrder : public Parameter
{
public:
    ParameterReverbEffectOrder(App *app, int paramId);
    virtual ~ParameterReverbEffectOrder();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterReverbEffectFeedDryIn : public Parameter
{
public:
    ParameterReverbEffectFeedDryIn(App *app, int paramId);
    virtual ~ParameterReverbEffectFeedDryIn();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterReverbEffectFeedWetOut : public Parameter
{
public:
    ParameterReverbEffectFeedWetOut(App *app, int paramId);
    virtual ~ParameterReverbEffectFeedWetOut();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterReverbEffectDryMix : public Parameter
{
public:
    ParameterReverbEffectDryMix(App *app, int paramId);
    virtual ~ParameterReverbEffectDryMix();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterReverbEffectDelayTime : public Parameter
{
public:
    ParameterReverbEffectDelayTime(App *app, int paramId);
    virtual ~ParameterReverbEffectDelayTime();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterReverbEffectDecay : public Parameter
{
public:
    ParameterReverbEffectDecay(App *app, int paramId);
    virtual ~ParameterReverbEffectDecay();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterBitCrushEffectRatio : public Parameter
{
public:
    ParameterBitCrushEffectRatio(App *app, int paramId);
    virtual ~ParameterBitCrushEffectRatio();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterSaturationEffectRatio : public Parameter
{
public:
    ParameterSaturationEffectRatio(App *app, int paramId);
    virtual ~ParameterSaturationEffectRatio();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterDelayEffectTime : public Parameter
{
public:
    ParameterDelayEffectTime(App *app, int paramId);
    virtual ~ParameterDelayEffectTime();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterDelayEffectFeedback : public Parameter
{
public:
    ParameterDelayEffectFeedback(App *app, int paramId);
    virtual ~ParameterDelayEffectFeedback();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterDelayEffectWetMix : public Parameter
{
public:
    ParameterDelayEffectWetMix(App *app, int paramId);
    virtual ~ParameterDelayEffectWetMix();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterVolumeEffectVolume : public Parameter
{
public:
    ParameterVolumeEffectVolume(App *app, int paramId);
    virtual ~ParameterVolumeEffectVolume();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterVolumeEffectMute : public Parameter
{
public:
    ParameterVolumeEffectMute(App *app, int paramId);
    virtual ~ParameterVolumeEffectMute();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

class ParameterEffectOn : public Parameter
{
public:
    ParameterEffectOn(App *app, int paramId);
    virtual ~ParameterEffectOn();

    void SetValue(ParameterProvider *provider, double value) override;
    void SetValue(double value) override;
    void UpdateValue() override;
};

#endif
