/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

// Author: Micha Hanselmann
// https://github.com/deermichel/reverb
// Many thanks to him to have made possible to re-use and modify his original reverb code.

#ifndef REVERB_H
#define REVERB_H

#include <vector>
using namespace std;

#include "Globals.h"
#include "CircularBuffer.h"

// See: https://ccrma.stanford.edu/realsimple/Reverb/Jot_s_FDN_Late_Reverberators.html
class Reverb
{
public:
    Reverb(int order, int maxNumSamples);
    virtual ~Reverb();

    void Process(vector<SAMPLE> *samples);

    void SetOrder(int order);
    
    void SetFeedDryIn(double feedDryIn);

    void SetFeedWetOut(double feedWetOut);

    void SetDrySignalMix(double drySignalMix);

    void SetDelayTapTime(int delayTime);

    void SetDecay(double decay);
    
protected:
    void Init();

    SAMPLE ProcessMainLine(int z);
    SAMPLE ProcessDelayLine(int z, int n);

    void GenerateHadamardMatrix(vector<vector<SAMPLE> > *result, double value);
        
    CircularBuffer<SAMPLE> _inputBuffer;
    
    vector<CircularBuffer<SAMPLE> > _delayBuffer;

    vector<vector<SAMPLE> >_feedbackMatrix;

    vector<double> _feedDryIn;

    vector<double> _feedWetOut;

    double _drySignalMix;

    vector<int> _delayTaps;

    vector<double> _decay;

    int _order;
    int _maxNumSamples;

    int _delayTime;
};

#endif
