/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#include "Scale.h"

double
Scale::DbToAmp(double db)
{
    double amp = pow(10.0, db/20.0);
    return amp;
}

double
Scale::AmpToDbNorm(double amp, double minDb, double maxDb)
{
    double db = 20.0*log10(fabs(amp));

    if (db < minDb)
        db = minDb;
    if (db > maxDb)
        db = maxDb;
        
    db = (db - minDb)/(maxDb - minDb);

    if (amp < 0.0)
        db = -db;
    
    return db;
}

double
Scale::HzPerBin(int fftSize, double sampleRate)
{
    return sampleRate/fftSize;
}

double
Scale::HzToMel(double freqHz)
{
    double mel = 2595.0*log10(1.0 + freqHz/700.0);

    return mel;
}

double
Scale::MelToHz(double freqMel)
{
    double hz = 700.0*(pow(10.0, (freqMel/2595.0)) - 1.0);
    
    return hz;
}

// Schlick sigmoid, see:
//
// https://dept-info.labri.u-bordeaux.fr/~schlick/DOC/gem2.ps.gz
//
// a included in [0, 1]
// a = 0.5 -> gives a line
double
Scale::Sigmoid(double t, double a)
{
    if (t < 0.0)
        t = 0.0;
    if (t > 1.0)
        t = 1.0;
    
    // sigmoid
    double f0 = (1.0/(1.0 - a) - 2.0)*(1.0 - 2.0*t);
    double ga;
    if (t < 0.5)
        ga = t/(f0 + 1.0);
    else
        ga = (f0 - t)/(f0 - 1.0);
    t = ga;
    
    return t;
}
