/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>

#include <algorithm>

#include "Serializer.h"

Serializer::Serializer()
{
    _file = NULL;
    _fileReadMode = true;
}

Serializer::~Serializer()
{
    CloseFile();
}

bool
Serializer::OpenReadFile(const char *fileName)
{
    _fileReadMode = true;
       
    _file = fopen(fileName, "rb");
    if (_file == NULL)
        return false;

    fseek(_file, 0, SEEK_END);
    long fsize = ftell(_file);
    fseek(_file, 0, SEEK_SET);

    _stream.resize(fsize);
    fread(_stream.data(), fsize, 1, _file);

    fclose(_file);
    _file = NULL;

    _readIndex = 0;
    
    return true;
}

bool
Serializer::OpenWriteFile(const char *fileName)
{
    _fileReadMode = false;
    
    _file = fopen(fileName, "wb");
        
    return (_file != NULL);
}

void
Serializer::CloseFile()
{
    if (_file != NULL)
    {
        if (!_fileReadMode)
            fwrite(_stream.data(), 1, _stream.size(), _file);
        fclose(_file);
    }
    
    _stream.resize(0);
    _file = NULL;
}

void
Serializer::PutBool(bool value)
{
    int ival = value;
    PutInt(ival);
}

void
Serializer::GetBool(bool *value)
{
    if (_readIndex + 4 > _stream.size())
        return;
    
    int ival;
    GetInt(&ival);
    *value = ival;
}

void
Serializer::PutInt(int value)
{
    _stream.push_back(((char *)&value)[0]);
    _stream.push_back(((char *)&value)[1]);
    _stream.push_back(((char *)&value)[2]);
    _stream.push_back(((char *)&value)[3]);
}

void
Serializer::GetInt(int *value)
{
    if (_readIndex + 4 > _stream.size())
        return;
    
    ((char *)value)[0] = _stream[_readIndex++];
    ((char *)value)[1] = _stream[_readIndex++];
    ((char *)value)[2] = _stream[_readIndex++];
    ((char *)value)[3] = _stream[_readIndex++];
}

void
Serializer::PutFloat(float value)
{
    _stream.push_back(((char *)&value)[0]);
    _stream.push_back(((char *)&value)[1]);
    _stream.push_back(((char *)&value)[2]);
    _stream.push_back(((char *)&value)[3]);
}

void
Serializer::GetFloat(float *value)
{
    if (_readIndex + 4 > _stream.size())
        return;
    
    ((char *)value)[0] = _stream[_readIndex++];     
    ((char *)value)[1] = _stream[_readIndex++];
    ((char *)value)[2] = _stream[_readIndex++];
    ((char *)value)[3] = _stream[_readIndex++];
}

void
Serializer::PutLong(long value)
{
    _stream.push_back(((char *)&value)[0]);
    _stream.push_back(((char *)&value)[1]);
    _stream.push_back(((char *)&value)[2]);
    _stream.push_back(((char *)&value)[3]);

    _stream.push_back(((char *)&value)[4]);
    _stream.push_back(((char *)&value)[5]);
    _stream.push_back(((char *)&value)[6]);
    _stream.push_back(((char *)&value)[7]);
}

void
Serializer::GetLong(long *value)
{
    if (_readIndex + 8 > _stream.size())
        return;
    
    ((char *)value)[0] = _stream[_readIndex++];
    ((char *)value)[1] = _stream[_readIndex++];
    ((char *)value)[2] = _stream[_readIndex++];
    ((char *)value)[3] = _stream[_readIndex++];
    ((char *)value)[4] = _stream[_readIndex++];
    ((char *)value)[5] = _stream[_readIndex++];
    ((char *)value)[6] = _stream[_readIndex++];
    ((char *)value)[7] = _stream[_readIndex++];
}

void
Serializer::PutDouble(double value)
{
    _stream.push_back(((char *)&value)[0]);
    _stream.push_back(((char *)&value)[1]);
    _stream.push_back(((char *)&value)[2]);
    _stream.push_back(((char *)&value)[3]);

    _stream.push_back(((char *)&value)[4]);
    _stream.push_back(((char *)&value)[5]);
    _stream.push_back(((char *)&value)[6]);
    _stream.push_back(((char *)&value)[7]);
}

void
Serializer::GetDouble(double *value)
{
    if (_readIndex + 8 > _stream.size())
        return;
    
    ((char *)value)[0] = _stream[_readIndex++];        
    ((char *)value)[1] = _stream[_readIndex++];    
    ((char *)value)[2] = _stream[_readIndex++];    
    ((char *)value)[3] = _stream[_readIndex++];
    ((char *)value)[4] = _stream[_readIndex++];        
    ((char *)value)[5] = _stream[_readIndex++];    
    ((char *)value)[6] = _stream[_readIndex++];    
    ((char *)value)[7] = _stream[_readIndex++];
}
