/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#include "SmoothSpace.h"

SmoothSpace::SmoothSpace() {}

SmoothSpace::~SmoothSpace() {}

void
SmoothSpace::Smooth(vector<SAMPLE> *data, int winSize)
{
    if (_win.size() != winSize)
        MakeWindow(winSize);

    _tmpBuf = *data;

    int halfWinSize = _win.size()/2;
    for (int i = 0; i < data->size(); i++)
    {
        SAMPLE sumVal = 0.0;
        SAMPLE sumCoeff = 0.0;
        for (int j = 0; j < winSize; j++)
        {
            int idx = i + j - halfWinSize;
            
            if ((idx < 0) || (idx > data->size() - 1))
                continue;
            
            SAMPLE val = _tmpBuf[idx];
            SAMPLE coeff = _win[j];
            
            sumVal += val*coeff;
            sumCoeff += coeff;
        }
        
        if (sumCoeff > 1e-15)
            sumVal /= sumCoeff;
    
        (*data)[i] = sumVal;
    }
}
    
void
SmoothSpace::MakeWindow(int size)
{
    _win.resize(size);
    
    // Hann
    for (int i = 0; i < _win.size(); i++)
        _win[i] = 0.5*(1.0 - cos(2.0*M_PI*
                                 ((double)i)/(_win.size() - 1)));
}
