/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "SamplesProvider.h"

class UISoundRect;
class Spectrogram;
class SpectroToSamples;
class Serializer;
class SoundRect : public SamplesProvider
{
public:
    SoundRect();
    virtual ~SoundRect();

    void RegisterUI(UISoundRect *uiRect);

    unsigned int GetId() const;
    
    UISoundRect *GetUI();

    void AssociateSamples(const vector<SAMPLE>& samples,
                          unsigned long nFrames,
                          unsigned int nChans);
    vector<SAMPLE> *GetSamples(int *nFrames, int *nChans);
    
    void AssociateSpectrogram(int chanNum, Spectrogram *spectro);
    Spectrogram *GetSpectrogram(int chanNum);
    
    void AssociateChromagram(int chanNum, Spectrogram *spectro);
    Spectrogram *GetChromagram(int chanNum);

    // SamplesProvider
    SamplesProvider *Clone() const override;
    void SetParamY(double paramY) override;
    void SetRange(double range) override;
    int GetNumSamples() const override;
    int GetNumChannels() const override;
    void GetSamples(vector<SAMPLE> *samples, double start, double end,
                    double timeStretch, double direction, double pitchShift) const override;
    int GetNumFrames() const override;

    int GetBlockMinSamples() const override;

    bool InternalPitchShift() const override;
    
    void Serialize(Serializer *serializer);
    void UnSerialize(Serializer *serializer);
    
protected:
    enum
    {
        SPECTROGRAM = 0,
        CHROMAGRAM
    };
    
    UISoundRect *_uiRect;

    static unsigned int _currentId;
    unsigned int _id;
    
    vector<SAMPLE> _samples;
    long _nFrames;
    int _nChans;

    Spectrogram *_spectro[2];
    Spectrogram *_chroma[2];

    SpectroToSamples *_spectroToSamples[2][2];
    
    double _paramY;
    double _range;

    bool _isClone;
};
