/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef SOURCE_H
#define SOURCE_H

#include "Globals.h"
#include "ParameterProvider.h"
#include "UISource.h"

class UISource;
class Serializer;
class EffectsBox;
class Envelope;
class Source : public ParameterProvider
{
public:
    enum SourceType
    {
        UNDEFINED_TYPE = -1,
        SINGLE_GRAIN,
        GRAIN_CLOUD,
        LOOP_SOURCE
    };

    Source();
    virtual ~Source();

    virtual Source *Clone() const = 0;
    
    SourceType GetSourceType() const;

    unsigned int GetId() const;
    
    void SetMidiNote(int midiNote);
    int GetMidiNote() const;

    void SetMidiPitchWheelValue(double pitch);
    double GetMidiPitchWheelValue() const;

    double GetTimeStretch() const;
    void SetTimeStretch(double stretch);

    EffectsBox *GetEffectsBox();

    void SetMuted(bool flag);
    bool IsMuted() const;

    Envelope *GetEnvelope();
    
    virtual void NextBuffer(vector<SAMPLE> *accumBuff, unsigned int numFrames) = 0;

    virtual void RegisterUI(UISource *uiSource) = 0;
    virtual UISource *GetUI() = 0;

    virtual void ResetCurrentParameterIndex() = 0;

    virtual void Serialize(Serializer *serializer);
    virtual void UnSerialize(Serializer *serializer);
    
protected:
    void Clone(Source *clone) const;
    
    double GetMidiNotePitch() const;
    
    SourceType _sourceType;

    static unsigned int _currentId;
    unsigned int _id;
    
    int _midiNote;
    double _midiPitchWheelValue;

    double _timeStretch;

    bool _isMuted;

    EffectsBox *_effectsBox;

    Envelope *_envelope;
    
    bool _isClone;
};

#endif
