/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Spectrogram.h"
#include "Complex.h"
#include "SpectroOverlapAddProcessor.h"

SpectroOverlapAddProcessor::SpectroOverlapAddProcessor(Spectrogram *spectro,
                                                       int chanNum)
{
    _spectro = spectro;
    _currentSpectroCol = 0;

    _chanNum = chanNum;
}

SpectroOverlapAddProcessor::~SpectroOverlapAddProcessor() {}

void
SpectroOverlapAddProcessor::ProcessFFT(vector<SAMPLE> *compBuf, int chanNum)
{
    if (chanNum != _chanNum)
        return;

    _spectro->SetCol(_currentSpectroCol, *compBuf);

    // Compute magns
    _tmpBuf.resize(compBuf->size()/4);
    Complex c;
    for (int i = 0; i < compBuf->size()/4; i ++)
    {
        c._re = (*compBuf)[i*2];
        c._im = (*compBuf)[i*2 + 1];
        SAMPLE magn = COMP_MAGN(c);

        _tmpBuf[i] = magn;
    }

    _spectro->SetMagnColDisp(_currentSpectroCol, _tmpBuf);
    
    _currentSpectroCol++;
}
