/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Spectrogram.h"
#include "SpectroToSamplesOverlapAddProcessor.h"

SpectroToSamplesOverlapAddProcessor::
SpectroToSamplesOverlapAddProcessor(Spectrogram *spectro)
{
    _spectro = spectro;
    _currentSpectroColNum = 0.0;

    _selFreq[0] = 0.0;
    _selFreq[1] = 1.0;

    _timeStretch = 1.0;
}

SpectroToSamplesOverlapAddProcessor::~SpectroToSamplesOverlapAddProcessor() {}

OverlapAddProcessor *
SpectroToSamplesOverlapAddProcessor::Clone()
{
    SpectroToSamplesOverlapAddProcessor *clone =
        new SpectroToSamplesOverlapAddProcessor(_spectro);
    clone->_currentSpectroColNum = _currentSpectroColNum;
    for (int i = 0; i < 2; i++)
        clone->_selFreq[i] = _selFreq[i];
    clone->_timeStretch = _timeStretch;
    
    return clone;
}

void
SpectroToSamplesOverlapAddProcessor::SetSpectroColNum(double colNum)
{
    _currentSpectroColNum = colNum;
}

void
SpectroToSamplesOverlapAddProcessor::SetSelFreq(double selFreq[2])
{
    _selFreq[0] = selFreq[0];
    _selFreq[1] = selFreq[1];
}

void
SpectroToSamplesOverlapAddProcessor::SetTimeStretch(double timeStretch)
{
    _timeStretch = timeStretch;
}

void
SpectroToSamplesOverlapAddProcessor::ProcessFFT(vector<SAMPLE> *compBuf, int chanNum)
{
    if (_currentSpectroColNum >= _spectro->GetNumCols())
        return;
    
    vector<SAMPLE> spectroCompBuf;
    _spectro->GetCol(_currentSpectroColNum, &spectroCompBuf);

    int selFreq[2] =
        { (int)(_selFreq[0]*spectroCompBuf.size()*0.5),
          (int)(_selFreq[1]*spectroCompBuf.size()*0.5) };
    for (int i = 0; i < spectroCompBuf.size(); i += 2)
    {
        if (i <= spectroCompBuf.size()*0.5)
            // First half
        {
            if ((i < selFreq[0]) || (i > selFreq[1]))
            {
                spectroCompBuf[i] = 0.0;
                spectroCompBuf[i + 1] = 0.0;
            }
        }
        else
            // Second half
        {
            if ((spectroCompBuf.size() - i < selFreq[0]) ||
                (spectroCompBuf.size() - i > selFreq[1]))
            {
                spectroCompBuf[i] = 0.0;
                spectroCompBuf[i + 1] = 0.0;
            }
        }
    }
    
    memcpy(compBuf->data(), spectroCompBuf.data(), spectroCompBuf.size()*sizeof(SAMPLE));

    _currentSpectroColNum += _timeStretch;
}
