/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef SPECTROGRAM_H
#define SPECTROGRAM_H

#include <vector>
using namespace std;

#include "Globals.h"

class Spectrogram
{
public:
    Spectrogram(int fftSize, int numCols);
    virtual ~Spectrogram();

    void SetCol(int colNum, const vector<SAMPLE> &compBuf);
    
    void GetCol(int colNum, vector<SAMPLE> *compBuf);
    // Get interpolated col
    void GetCol(double colNum, vector<SAMPLE> *compBuf);

    int GetFFTSize() const;
    int GetNumCols() const;

    // magn data for display
    void SetMagnColDisp(int colNum, const vector<SAMPLE> &magnBuf);
    const vector<SAMPLE> *GetMagnDataDisp();
    
    void NormalizeMagnsDisp();

    void PreProcessPhases();
    
protected:
    void SetMagnCol(int colNum, const vector<SAMPLE> &magnBuf);
    void GetMagnCol(int colNum, vector<SAMPLE> *magnBuf);
    
    void SetPhaseCol(int colNum, const vector<SAMPLE> &phaseBuf);
    void GetPhaseCol(int colNum, vector<SAMPLE> *phaseBuf);
    
    double FmodNegative(double x, double y);
    void FindClosestPhase(double *phase, double refPhase);
    
    int _fftSize;
    int _numCols;

    vector<SAMPLE> _compBuf;
    vector<SAMPLE> _magnBuf;
    vector<SAMPLE> _phaseBuf;

    vector<SAMPLE> _magnBufDisp;
    
    vector<SAMPLE> _tmpMagnBuf;
    vector<SAMPLE> _tmpPhaseBuf;
};

#endif
