/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Parameter.h"
#include "StereoWidener.h"
#include "DSP.h"
#include "StereoEffect.h"

#define WIDTH_BOOST_FACTOR 5.0

StereoEffect::StereoEffect()
{
    _pan = 0.0;
    _width = 1.0;

    _widener = new StereoWidener();

    _params[ON_PARAM] = new ParameterEffectOn(NULL, ON_PARAM);
    _params[PAN_PARAM] = new ParameterStereoEffectPan(NULL, PAN_PARAM);
    _params[WIDTH_PARAM] = new ParameterStereoEffectWidth(NULL, WIDTH_PARAM);
}

StereoEffect::~StereoEffect()
{
    delete _widener;

    for (int i = 0; i < NUM_PARAMS; i++)
        delete _params[i];
}

void
StereoEffect::Process(vector<SAMPLE> *buffer, int nFrames, int nChan)
{
    if (nChan < 2)
        return;

    UpdateParameters();

    double on = _params[ON_PARAM]->GetValue();
    if (on < 0.5)
        return;
    
    _widener->Process(buffer, nFrames, nChan, _width);
    
    DSP::Balance(buffer, nChan, _pan);
}

void
StereoEffect::SetPan(double pan)
{
    _pan = pan;
}

void
StereoEffect::SetWidth(double width)
{
    _width = width;
}

Parameter *
StereoEffect::GetParameter(ParamIdx paramIdx)
{
    return _params[paramIdx];
}

void
StereoEffect::Serialize(Serializer *serializer)
{
    for (int i = 0; i < NUM_PARAMS; i++)
        _params[i]->Serialize(serializer);
}

void
StereoEffect::UnSerialize(Serializer *serializer)
{
    for (int i = 0; i < NUM_PARAMS; i++)
        _params[i]->UnSerialize(serializer);

    UpdateParameters();
}

void
StereoEffect::UpdateParameters()
{
    _pan = _params[PAN_PARAM]->GetValue();
    _pan = _pan*0.01;
    
    _width = _params[WIDTH_PARAM]->GetValue();
    _width = _width*0.01;

    _width = _width + 1.0;
    
    // Stereo width boost
    if (_width > 1.0)
    {
        double t = (_width - 1.0);
        _width = (1.0 - t)*_width + t*_width*WIDTH_BOOST_FACTOR*0.5;
    }
}
