/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "StereoWidener.h"

StereoWidener::StereoWidener()
{
    _angle0 = ComputeAngle0();
    _angle1 = ComputeAngle1();
}

StereoWidener::~StereoWidener() {}

void
StereoWidener::Process(vector<SAMPLE> *buffer, int nFrames, int nChan, double widthFactor)
{
    if (nChan < 2)
        return;
    
    for (int i = 0; i < nFrames; i++)
    {
        // Compute
        SAMPLE l = (*buffer)[i*2];
        SAMPLE r = (*buffer)[i*2 + 1];
        
        StereoWiden(&l, &r, widthFactor, _angle0, _angle1);
        
        (*buffer)[i*2] = l;
        (*buffer)[i*2 + 1] = r;
    }
}

Complex
StereoWidener::ComputeAngle0()
{
    SAMPLE magn = 1.0;
    SAMPLE phase = -M_PI/4.0;

    Complex result;
    MAGN_PHASE_COMP(magn, phase, result);

    return result;
}

Complex
StereoWidener::ComputeAngle1()
{
    SAMPLE magn = 1.0;
    SAMPLE phase = M_PI/4.0;

    Complex result;
    MAGN_PHASE_COMP(magn, phase, result);
    
    return result;
}

void
StereoWidener::StereoWiden(SAMPLE *l, SAMPLE *r,
                           double widthFactor,
                           const Complex &angle0,
                           const Complex &angle1)
{    
    // Init
    Complex signal0;
    signal0._re = *l;
    signal0._im = *r;
    
    // Rotate
    Complex signal1;
    COMP_MULT(signal0, angle0, signal1);
    
    // Scale over y
    signal1._im *= widthFactor;
    
    // Rotate back
    Complex signal2;
    COMP_MULT(signal1, angle1, signal2);
    
    // Result
    *l = signal2._re;
    *r = signal2._im;
}
