/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef STEREOIZER_H
#define STEREOIZER_H

#include <vector>
using namespace std;

#include "Globals.h"

// Implementation of:
// "Downmix-compatible conversion from mono to stereo in time and frequency-domain"
// by Marco Fink
//
// See: https://www.researchgate.net/publication/283008516_Downmix-compatible_conversion_from_mono_to_stereo_in_time-_and_frequency-domain
//
// TODO: simple delay + substraction for right channel instead of using a full convolution

class Convolver;
class Stereoizer
{
public:
    Stereoizer(double width = 1.0);
    virtual ~Stereoizer();

    void Reset();
    
    void SetWidth(double width);
    
    int GetLatency();
    
    void Process(const vector<SAMPLE> &mono,
                 vector<SAMPLE> *leftChan,
                 vector<SAMPLE> *rightChan);
    
protected:
    void GenerateIRs(vector<SAMPLE> *irl, vector<SAMPLE> *irr);
    
    void AdjustGain(vector<SAMPLE> *samples);

    void ComputeIFFT(const vector<SAMPLE> &inReal, vector<SAMPLE> *outReal);
    
    double _width;

    Convolver *_convolverL;
    Convolver *_convolverR;
};

#endif
