/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <math.h>

#ifdef _MSC_VER
#include <windows.h>
#endif

// Graphics includes
#ifdef __MACOSX_CORE__
#include <GLUT/glut.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#include "UIScreen.h"
#include "UIStyle.h"
#include "UIBufferCanvas.h"

UIBufferCanvas::UIBufferCanvas(UIScreen *screen,
                               UIStyle *style,
                               const char *title,
                               double x, double y,
                               double width, double height,
                               double dataLo, double dataHi,
                               int buffSize)
: UICanvas(screen, style, title, x, y, width, height)
{
    _buffSize = buffSize;
    _buffer.resize(_buffSize);

    _dataLo = dataLo;
    _dataHi = dataHi;
}

UIBufferCanvas::~UIBufferCanvas() {}

void
UIBufferCanvas::DrawContents()
{
    UICanvas::DrawContents();
        
    // Draw title box separator    
    glPushMatrix();

    glLoadIdentity(); // for non-zoomable
    
    glBegin(GL_LINES);

    double t = 0.0;
    double tStep = 1.0/_buffSize;
    for (int i = 0; i < _buffSize - 1; i++)
    {
        double amp0 = _buffer[i];
        double amp1 = _buffer[i + 1];

        amp0 = (amp0 - _dataLo)/(_dataHi - _dataLo);
        amp1 = (amp1 - _dataLo)/(_dataHi - _dataLo);

        amp0 = 1.0 - amp0;
        amp1 = 1.0 - amp1;
        
        glVertex3f((_x + t*_width)*_screen->GetWidth(),
                   (1.0 - (_y + amp0*_height))*_screen->GetHeight(),
                   0.0);
                
        glVertex3f((_x + (t + tStep)*_width)*_screen->GetWidth(),
                   (1.0 - (_y + amp1*_height))*_screen->GetHeight(),
                   0.0);

        t += tStep;
    }
    
    glEnd();

    glPopMatrix();
}
