/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef UI_CANVAS_H
#define UI_CANVAS_H

class UIScreen;
class UIStyle;
class UICanvas
{
public:
    UICanvas(UIScreen *screen,
             UIStyle *style,
             const char *title,
             double x, double y,
             double width, double height);
    virtual ~UICanvas();
    
    virtual void Draw();

    virtual void SetVisible(bool flag);
    
    void UpdateStyle(UIStyle *style);

    void GetContentsRect(double *x, double *y,
                         double *width, double *height);
        
    virtual void OnMouse(int button, int state, double x, double y);
    virtual void OnMouseDrag(double x, double y, double fineDragFactor = 1.0);

    virtual bool Pick(double x, double y) const;
    
protected:
    virtual void OnMouseDragNorm(double nx, double ny,
                                 double dx, double dy, bool inside);
        
    void DrawBorder();    
    virtual void DrawContents();
    void DrawTitle();
      
    void ComputeCorners(double corners[4][2]);
    
    UIScreen *_screen;
    UIStyle *_style;

    char _title[255];
    
    double _x;
    double _y;
    
    double _width;
    double _height;

    // Mouse
    bool _mouseDown;
    double _prevDragX;
    double _prevDragY;

    bool _visible;

    bool _drawBorder;
    bool _centerTitle;
};

#endif

