/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#ifdef _MSC_VER
#include <windows.h>
#endif

#ifdef __MACOSX_CORE__
#include <GLUT/glut.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#include "Globals.h"
#include "Parameter.h"
#include "UIScreen.h"
#include "UIStyle.h"
#include "UICheckboxWidget.h"

UICheckboxWidget::UICheckboxWidget(UIScreen *screen,
                                   UIStyle *style,
                                   const char *title,
                                   double x, double y,
                                   double width, double height)
: UIEffectWidget(screen, style, title, x, y, width, height) {}

UICheckboxWidget::~UICheckboxWidget() {}

void
UICheckboxWidget::OnMouse(int button, int state, double x, double y)
{
    UIEffectWidget::OnMouse(button, state, x, y);

    if (_leftMouseDown)
    {
        double value = _param->GetValue();
        if (value <= 0.5)
            value = 1.0;
        else if (value > 0.5)
            value = 0.0;
        _param->SetValue(value);
    }
}

void
UICheckboxWidget::DrawWidget()
{
    glPushMatrix();

    glLoadIdentity(); // for non-zoomable

    double size = _width*_screen->GetWidth()*0.5;
    glTranslatef(_x*_screen->GetWidth() + size,
                 (1.0 - _y)*_screen->GetHeight() - size, 0.0);

    double lineWidth = _style->GetParamLineWidth();
    
    // Draw a square
    glLineWidth(lineWidth);
    
    glBegin(GL_LINE_LOOP);
    glVertex3f(-size, -size,  0.0);
    glVertex3f( size, -size,  0.0);
    glVertex3f( size,  size,  0.0);
    glVertex3f(-size,  size,  0.0);
    glEnd();
    
    if (_param->GetValue() >= 1.0)
    {
        // If checked, draw a cross
        double crossSize = size*0.8;
        
        glBegin(GL_LINES);
        glVertex3f(-crossSize, -crossSize, 0.0);
        glVertex3f( crossSize,  crossSize, 0.0);
        glEnd();
        
        glBegin(GL_LINES);
        glVertex3f(-crossSize, crossSize,  0.0);
        glVertex3f( crossSize, -crossSize, 0.0);
        glEnd();
    }
    
    glPopMatrix();
}
