/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Parameter.h"
#include "UIEffectWidget.h"
#include "UIKnobWidget.h"
#include "UICheckboxWidget.h"
#include "UIScreen.h"
#include "UIDelayEffect.h"

UIDelayEffect::UIDelayEffect(UIScreen *screen,
                             UIStyle *style,
                             const char *title,
                             double x, double y,
                             double width, double height)
: UIEffect(screen, style, title, x, y, width, height, DelayEffect::NUM_PARAMS)
{ 
    _centerTitle = true;
    _drawBorder = true;
    
    GenerateContents();
}

UIDelayEffect::~UIDelayEffect() {}
    
void
UIDelayEffect::GenerateContents()
{
    double whRatio = _screen->GetWidth()/_screen->GetHeight();
    
    double knobSize = _width/6;
    UIKnobWidget *timeWidget = new UIKnobWidget(_screen, _style,
                                                "Time",
                                                _x + 0.25*_width - knobSize/2,
                                                _y + _height/2 - knobSize,
                                                knobSize, knobSize*whRatio);
    _widgets[DelayEffect::TIME_PARAM] = timeWidget;

    UIKnobWidget *feedbackWidget = new UIKnobWidget(_screen, _style,
                                                    "Feedback",
                                                    _x + 0.5*_width - knobSize/2,
                                                    _y + _height/2 - knobSize,
                                                    knobSize, knobSize*whRatio);
    _widgets[DelayEffect::FEEDBACK_PARAM] = feedbackWidget;

    UIKnobWidget *dryWetWidget = new UIKnobWidget(_screen, _style,
                                                  "Dry/Wet",
                                                  _x + 0.75*_width - knobSize/2,
                                                  _y + _height/2 - knobSize,
                                                  knobSize, knobSize*whRatio);
    _widgets[DelayEffect::WET_MIX_PARAM] = dryWetWidget;
}
