/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Parameter.h"
#include "UIEffectWidget.h"
#include "UICheckboxWidget.h"
#include "UIScreen.h"
#include "UIEffect.h"

UIEffect::UIEffect(UIScreen *screen,
                   UIStyle *style,
                   const char *title,
                   double x, double y,
                   double width, double height,
                   int numParams)
: UICanvas(screen, style, title, x, y, width, height)
{
    _widgets.resize(numParams);
    
    _centerTitle = true;

    GenerateContents();
}

UIEffect::~UIEffect()
{
    for (int i = 0; i < _widgets.size(); i++)
        delete _widgets[i];
}

void
UIEffect::SetVisible(bool flag)
{
    UICanvas::SetVisible(flag);
    
    for (int i = 0; i < _widgets.size(); i++)
        _widgets[i]->SetVisible(flag);
}

void
UIEffect::OnMouse(int button, int state, double x, double y)
{
    UICanvas::OnMouse(button, state, x, y);

    for (int i = 0; i < _widgets.size(); i++)
        _widgets[i]->OnMouse(button, state, x, y);
}

void
UIEffect::OnMouseDrag(double x, double y, double fineDragFactor)
{
    UICanvas::OnMouseDrag(x, y, fineDragFactor);

    for (int i = 0; i < _widgets.size(); i++)
        _widgets[i]->OnMouseDrag(x, y, fineDragFactor);
}

void
UIEffect::SetParameter(int idx, Parameter *param)
{
    _widgets[idx]->SetParameter(param);
}

void
UIEffect::GenerateContents()
{
#define ON_CHECKBOX_SIZE   0.02
#define ON_CHECKBOX_MARGIN 0.01

    double whRatio = _screen->GetWidth()/_screen->GetHeight();
    
    // Create the "on/off" checkbox in the corner
    UICheckboxWidget *onWidget =
        new UICheckboxWidget(_screen, _style,
                             "",
                             _x + _width - ON_CHECKBOX_SIZE - ON_CHECKBOX_MARGIN,
                             _y + ON_CHECKBOX_MARGIN*whRatio,
                             ON_CHECKBOX_SIZE, ON_CHECKBOX_SIZE*whRatio);
    _widgets[UIEffect::ON_PARAM] = onWidget;
}

void
UIEffect::DrawContents()
{
    for (int i = 0; i < _widgets.size(); i++)
        _widgets[i]->Draw();
}

void
UIEffect::UpdateParameters()
{
    for (int i = 0; i < _widgets.size(); i++)
    {
        double val = _widgets[i]->GetParameter()->GetValue();
        _widgets[i]->GetParameter()->SetValue(val);
    }
}
