/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>

#ifdef _MSC_VER
#include <windows.h>
#endif

#ifdef __MACOSX_CORE__
#include <GLUT/glut.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#include "Globals.h"
#include "Parameter.h"
#include "UIStyle.h"
#include "UIScreen.h"
#include "UIEffectWidget.h"

#define TEXT_HEIGHT 0.0434

UIEffectWidget::UIEffectWidget(UIScreen *screen,
                               UIStyle *style,
                               const char *title,
                               double x, double y,
                               double width, double height)
: UICanvas(screen, style, "", x, y, width, height)
{
    sprintf(_title, title);

    _param = NULL;

    _leftMouseDown = false;
}

UIEffectWidget::~UIEffectWidget() {}

void
UIEffectWidget::SetParameter(Parameter *param)
{
    _param = param;

    _param->UpdateValue();
}

Parameter *
UIEffectWidget::GetParameter()
{
    return _param;
}

void
UIEffectWidget::Draw()
{
    UICanvas::Draw();
    
    DrawWidget();
    DrawTitle();
    DrawValue();
}

void
UIEffectWidget::OnMouse(int button, int state, double x, double y)
{
    //UICanvas::OnMouse(button, state, x, y);
    
    if (Pick(x, y))
    {
        if ((button == GLUT_RIGHT_BUTTON) && (state == GLUT_DOWN))
        {
            double defaultVal = _param->GetDefaultValue();
            _param->SetValue(defaultVal);
        }

        if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN))
            _leftMouseDown = true;

        _prevDragX = x;
        _prevDragY = y;
    }

    if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_UP))
        _leftMouseDown = false;
}

void
UIEffectWidget::DrawTitle()
{
    // Draw title label
    double a = 0.8f + 0.2*sin(0.5*M_PI*Time::Instance().GetSeconds());
    
    double col[4];
    _style->GetUIWindowTitleColor(col);
    glColor4f(col[0], col[1], col[2], a*col[3]);
    
    GLint len = strlen(_title);

    // Compute string length in pixels
    double lenPix = glutStrokeLength(GLUT_STROKE_MONO_ROMAN, (const unsigned char *)_title);
    
    glPushMatrix();

    glLoadIdentity(); // for non-zoomable

    glTranslatef((_x + _width*0.5)*_screen->GetWidth()
                 - lenPix*0.5*0.1,
                 (1.0 - _y + TEXT_HEIGHT*0.5)*
                 _screen->GetHeight(),
                 0.0);
    glScalef(0.1, 0.1, 0.1);
    
    for(int i = 0; i < len; i++)
        glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, _title[i]);
    
    glPopMatrix();
}

void
UIEffectWidget::DrawValue()
{
    // Draw title label
    double a = 0.8f + 0.2*sin(0.5*M_PI*Time::Instance().GetSeconds());
    
    double col[4];
    _style->GetUIWindowTitleColor(col);
    glColor4f(col[0], col[1], col[2], a*col[3]);

    const char *valueStr = _param->GetValueStr();
    
    GLint len = strlen(valueStr);

    // Compute string length in pixels
    double lenPix = glutStrokeLength(GLUT_STROKE_MONO_ROMAN,
                                    (const unsigned char *)valueStr);
    
    glPushMatrix();

    glLoadIdentity(); // for non-zoomable

    glTranslatef((_x + _width*0.5)*_screen->GetWidth()
                 - lenPix*0.5*0.1,
                 (1.0 - _y - TEXT_HEIGHT - _height)*
                 _screen->GetHeight(),
                 0.0);
    glScalef(0.1, 0.1, 0.1);
    
    for(int i = 0; i < len; i++)
        glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, valueStr[i]);
    
    glPopMatrix();
}
