/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>

#include "Globals.h"
#include "App.h"
#include "Source.h"
#include "Envelope.h"
#include "UIScreen.h"
#include "UIStyle.h"
#include "UIEnvelopeTypeCanvas.h"

UIEnvelopeTypeCanvas::UIEnvelopeTypeCanvas(App *app,
                                           UIScreen *screen,
                                           UIStyle *style,
                                           const char *title,
                                           double x, double y,
                                           double width, double height)
: UIBufferCanvas(screen, style, title,
                 x, y, width, height,
                 0.0, 1.0,
                 ENVELOPE_LEN)
{
    _app = app;
    
    _envelopeType = 0;

    GenerateContents();
}

UIEnvelopeTypeCanvas::~UIEnvelopeTypeCanvas() {}

void
UIEnvelopeTypeCanvas::Draw()
{
    UICanvas::Draw();

    if (_envelopeType == Envelope::HAND_DRAWN)
        DrawBorder();
}


void
UIEnvelopeTypeCanvas::SetEnvelopeType(int envelopeType)
{
    _envelopeType = envelopeType;
    GenerateContents();
}

void
UIEnvelopeTypeCanvas::OnMouseDragNorm(double x, double y,
                                      double dx, double dy, bool inside)
{
    Source *source = _app->GetCurrentSource();
    if (source == NULL)
        return;
    Envelope *envelope = source->GetEnvelope();
    
    if (_envelopeType != Envelope::HAND_DRAWN)
        return;
    
    if (!_mouseDown)
        return;
    
    if (x < 0.0)
        x = 0.0;
    if (x > 1.0)
        x = 1.0;
    if (y < 0.0)
        y = 0.0;
    if (y > 1.0)
        y = 1.0;
    
    y = y*0.5;
    dy = dy*0.5;
    
    const vector<SAMPLE> &env = envelope->GetEnvelope(_envelopeType);
    _buffer = env;
    
    int envSize = envelope->GetEnvelopeSize();

    double x0 = x - dx;
    double x1 = x;

    double y0 = y - dy;
    double y1 = y;
    
    if (x0 > x1)
    {
        double tmp0 = x0;
        x0 = x1;
        x1 = tmp0;

        double tmp1 = y0;
        y0 = y1;
        y1 = tmp1;
    }
    
    if (y0 > 0.5)
        y0 = 1.0;
    else
        y0 *= 2.0;
    
    if (y1 > 0.5)
        y1 = 1.0;
    else
        y1 *= 2.0;
    
    int idx0 = (int)(x0*envSize);
    if (idx0 < 0)
        idx0 = 0;
    if (idx0 > envSize - 1)
        idx0 = envSize - 1;

    int idx1 = (int)(x1*envSize);
    if (idx1 < 0)
        idx1 = 0;
    if (idx1 > envSize - 1)
        idx1 = envSize - 1;

    for (int i = idx0; i <= idx1; i++)
    {
        double t = 1.0;
        if (idx0 != idx1)
            t = ((double)i - idx0)/(idx1 - idx0 + 1);
        
        double yy = y0 + t*(y1 - y0);

        _buffer[i] = 1.0 - yy;
    }

    envelope->SetEnvelope(_envelopeType, _buffer);
    
    GenerateContents();
}

void
UIEnvelopeTypeCanvas::GenerateContents()
{
    Source *source = _app->GetCurrentSource();
    if (source == NULL)
        return;
    Envelope *envelope = source->GetEnvelope();
    
    const vector<SAMPLE> &env = envelope->GetEnvelope(_envelopeType);
    _buffer = env;
}
