/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>

#ifdef _MSC_VER
#include <windows.h>
#endif

#ifdef __MACOSX_CORE__
#include <GLUT/glut.h>
#else
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#include "Time.h"
#include "UIStyle.h"
#include "UIScreen.h"
#include "UIGrain.h"

UIGrain::~UIGrain() {}

UIGrain::UIGrain(UIStyle *style, UIScreen *screen,
                 double x, double y)
{
    _style = style;
    _screen = screen;
    
    _x = x;
    _y = y;
    
    _defSize = 10.0f;
    _size = _defSize;
    _onSize = 30.0f;
    _isOn = false;
    _firstTrigger = false;
    _startTime = Time::Instance().GetSeconds();
    _triggerTime = 0.0;
}

// Draw method
void
UIGrain::Draw()
{
    if (!_isOn)
        return;
    
    double w = _screen->GetWidth();
    double h = _screen->GetHeight();
    
    double grainVisColor[4];
    _style->GetGrainDrawColor(grainVisColor);
    double colR0 = grainVisColor[0];
    double colG0 = grainVisColor[1];
    double colB0 = grainVisColor[2];
    double colA0 = grainVisColor[3];

    double grainVisDefColor[4];
    _style->GetGrainDrawDefColor(grainVisDefColor);
    double defR0 = grainVisDefColor[0];
    double defG0 = grainVisDefColor[1];
    double defB0 = grainVisDefColor[2];
    double defA0 = grainVisDefColor[3];
    
    double colR;
    double colG;
    double colB;
    double colA;
    
    double t_sec = Time::Instance().GetSeconds() - _triggerTime;
    if (_firstTrigger)
    {
        // Slew size
        double mult = 0.0;
        if (_isOn)
        {
            mult = exp(-t_sec/(0.8*_durSec));
            
            colR = mult*colR0;
            colG = mult*colG0;
            colB = mult*colB0;
            colA = mult*colA0;

            _size = _defSize + (1.0 - mult)*(_onSize - _defSize);

            if (colB < 0.001)
                _isOn = false;
        }
    }
    
    glColor4f(colR,colG,colB,colA);
    
    // disk version - for graphics cards that don't support GL_POINT_SMOOTH
    //       glTranslatef((GLdouble)_x,(GLdouble)_y,0.0);
    //       gluDisk(gluNewQuadric(),0,_size*0.5f, 64,1);
    
    // end disk version
    
    
    // Point version (preferred - better quality)
    double prevSize = 1.0f;
    glGetDoublev(GL_POINT_SIZE, &prevSize);
    glPointSize(_size);
    glBegin(GL_POINTS);
    glVertex3f(_x*w, _y*h, 0.0);
    glEnd();
    glPointSize(prevSize);
}

void
UIGrain::Trigger(double dur)
{
    _isOn = true;
    if (!_firstTrigger)
        _firstTrigger = true;
    _durSec = dur*0.001;
    _triggerTime = Time::Instance().GetSeconds();
}

void
UIGrain::MoveTo(double x, double y)
{
    _x = x;
    _y = y;
}

double
UIGrain::GetX()
{
    return _x;
}

double
UIGrain::GetY()
{
    return _y;
}

void
UIGrain::UpdateStyle(UIStyle *style)
{
    _style = style;
}
