/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <math.h>

#include "Globals.h"
#include "UIScreen.h"
#include "UIStyle.h"
#include "UILFOCanvas.h"

UILFOCanvas::UILFOCanvas(UIScreen *screen,
                         UIStyle *style,
                         const char *title,
                         double x, double y,
                         double width, double height)
: UIBufferCanvas(screen, style, title, x, y, width, height)
{
    _freq = 1.0;
    _amp = 0.5;

    GenerateContents();
}

UILFOCanvas::~UILFOCanvas() {}

void
UILFOCanvas::SetFreq(double freq)
{
    if (fabs(freq - _freq) > 1e-15)
    {
        _freq = freq;
        GenerateContents();
    }
}

void
UILFOCanvas::SetAmp(double amp)
{
    if (fabs(amp - _amp) > 1e-15)
    {
        _amp = amp;
        GenerateContents();
    }
}

void
UILFOCanvas::GenerateContents()
{
    // 5 seconds
#define TIME_FRAME 10.0
    
    double t = 0.0;
    double tStep = 1.0/(_buffSize - 1);
    for (int i = 0; i < _buffSize; i++)
    {
        _buffer[i] = _amp*sin(t*_freq*2.0*M_PI*TIME_FRAME);
        t += tStep;
    }
}
