/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef UI_PARAMETER_H
#define UI_PARAMETER_H

class UIStyle;
class UIScreen;
class Parameter;
class UIParameter
{
public:
    UIParameter(Parameter *param, UIScreen *screen, UIStyle *style);
    virtual ~UIParameter();

    void Draw();
    bool Pick(double x, double y);
    void SetPosition(double x, double y);

    double GetPosX() const;
    double GetPosY() const;

    void TranslateX(double dx);
    void TranslateY(double dy);
    
    void SetName(const char *name);
    const char *GetName() const;
    
    void SetValueStr(const char *valueStr);
    const char *GetValueStr() const;

    void DrawPieMenuLink(double menuCenterX,
                         double menuCenterY,
                         double menuCenterOffset);

    void UpdateStyle(UIStyle *style);

    void Serialize(Serializer *serializer);
    
    void UnSerialize(Serializer *serializer);
        
protected:
    void DrawBackground();
    void DrawBorder();
    void DrawLabel();
    void DrawMeter();

    char _name[255];
    char _valueStr[255];

    double _x;
    double _y;

    Parameter *_param;
    UIScreen *_screen;
    UIStyle *_style;
};

#endif
