/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Parameter.h"
#include "UIEffectWidget.h"
#include "UIKnobWidget.h"
#include "UIScreen.h"
#include "ReverbEffect.h"
#include "UIReverbEffect.h"

UIReverbEffect::UIReverbEffect(UIScreen *screen,
                               UIStyle *style,
                               const char *title,
                               double x, double y,
                               double width, double height)
: UIEffect(screen, style, title, x, y, width, height, ReverbEffect::NUM_PARAMS)
{    
    _centerTitle = true;
    _drawBorder = true;
    
    GenerateContents();
}

UIReverbEffect::~UIReverbEffect() {}
    
void
UIReverbEffect::GenerateContents()
{
    double whRatio = _screen->GetWidth()/_screen->GetHeight();

    //double knobSize = _width/12;
    double knobSize = _height/6;
    
    double step = 1.0/7.0;
    
    UIKnobWidget *orderWidget = new UIKnobWidget(_screen, _style,
                                                 "Order",
                                                 _x + step*_width - knobSize/2,
                                                 _y + _height/2 - knobSize,
                                                 knobSize, knobSize*whRatio);
    _widgets[ReverbEffect::ORDER_PARAM] = orderWidget;

    UIKnobWidget *feedDryInWidget = new UIKnobWidget(_screen, _style,
                                                     "Dry In",
                                                     _x + step*2.0*_width - knobSize/2,
                                                     _y + _height/2 - knobSize,
                                                     knobSize, knobSize*whRatio);
    _widgets[ReverbEffect::FEED_DRY_IN_PARAM] = feedDryInWidget;

    UIKnobWidget *feedWetOutWidget = new UIKnobWidget(_screen, _style,
                                                      "Wet Out",
                                                      _x + step*3.0*_width - knobSize/2,
                                                      _y + _height/2 - knobSize,
                                                      knobSize, knobSize*whRatio);
    _widgets[ReverbEffect::FEED_WET_OUT_PARAM] = feedWetOutWidget;

    UIKnobWidget *dryMixWidget = new UIKnobWidget(_screen, _style,
                                                  "Dry Mix",
                                                  _x + step*4.0*_width - knobSize/2,
                                                  _y + _height/2 - knobSize,
                                                  knobSize, knobSize*whRatio);
    _widgets[ReverbEffect::DRY_MIX_PARAM] = dryMixWidget;

    UIKnobWidget *delayTimeWidget = new UIKnobWidget(_screen, _style,
                                                     "Delay",
                                                     _x + step*5.0*_width - knobSize/2,
                                                     _y + _height/2 - knobSize,
                                                     knobSize, knobSize*whRatio);
    _widgets[ReverbEffect::DELAY_TIME_PARAM] = delayTimeWidget;

    UIKnobWidget *decayWidget = new UIKnobWidget(_screen, _style,
                                                 "Decay",
                                                 _x + step*6.0*_width - knobSize/2,
                                                 _y + _height/2 - knobSize,
                                                 knobSize, knobSize*whRatio);
    _widgets[ReverbEffect::DECAY_PARAM] = decayWidget;
}
