/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "Parameter.h"
#include "UIEffectWidget.h"
#include "UIKnobWidget.h"
#include "UICheckboxWidget.h"
#include "UIScreen.h"
#include "UISaturationEffect.h"

UISaturationEffect::UISaturationEffect(UIScreen *screen,
                                       UIStyle *style,
                                       const char *title,
                                       double x, double y,
                                       double width, double height)
: UIEffect(screen, style, title, x, y, width, height, SaturationEffect::NUM_PARAMS)
{ 
    _centerTitle = true;
    _drawBorder = true;
    
    GenerateContents();
}

UISaturationEffect::~UISaturationEffect() {}
    
void
UISaturationEffect::GenerateContents()
{
    double whRatio = _screen->GetWidth()/_screen->GetHeight();
    
    //double knobSize = _width/6;
    double knobSize = _width/3;
    UIKnobWidget *ratioWidget = new UIKnobWidget(_screen, _style,
                                                 "Ratio",
                                                 _x + 0.5*_width - knobSize/2,
                                                 _y + _height/2 - knobSize,
                                                 knobSize, knobSize*whRatio);
    _widgets[SaturationEffect::RATIO_PARAM] = ratioWidget;
}
