/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef UI_SINGLE_GRAIN_H
#define UI_SINGLE_GRAIN_H

#include <vector>
using namespace std;

#include "UISource.h"

class UIStyle;
class UIScreen;
class SoundRect;
class UIGrain;
class Serializer;
class SingleGrain;
class UISingleGrain : public UISource
{
public:
    UISingleGrain(SingleGrain *grain,
                  UIStyle *_style,
                  UIScreen *screen,
                  double x, double y,
                  vector<SoundRect *> *rects);
    
    virtual ~UISingleGrain();
    
    void Draw();
    
    void GetTriggerPos(double *playPos,
                       double *playVols,
                       double dur);

    void UpdatePosition(double x, double y) override;
    void SetState(bool on);

    void SetMuted(bool flag);
    
    void SetSelectState(bool state);

    bool Pick(double x, double y) override;
    
    double GetX() override;
    double GetY() override;
    double GetRadius();
    
    void SetDuration(double dur);

    void UpdateStyle(UIStyle *style);

    bool PickParameters(double x, double y) override;

    void Serialize(Serializer *serializer) override;
    void UnSerialize(Serializer *serializer) override;
    
protected:
    void UpdateGrainPosition(double x, double y);
        
    struct Point
    {
        double _x;
        double _y;
    };
    
    bool _isOn;

    bool _isSelected;
    double _startTime;
    
    double _freq;
    
    double _x;
    double _y;
    
    double _selRad;
    double _lambda;
    
    double _maxSelRad;
    double _minSelRad;
    double _targetRad;
    
    // Grain voice visualizations
    UIGrain *_uiGrain;
    
    // Registered sound rectangles 
    vector<SoundRect *> *_rects;

    UIStyle *_style;
    UIScreen *_screen;

    SingleGrain *_grain;
};

#endif
