/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#ifndef UI_STYLE_H
#define UI_STYLE_H

class UIStyle
{
public:
    UIStyle() {}
    virtual ~UIStyle() {}

    void GetAppName(char appName[512]);
    
    void GetBackgroundColor(double col[4]) const;

    double GetAxisLineWidth() const;
    void GetAxisXColor(double col[4]) const;
    void GetAxisYColor(double col[4]) const;
    void GetAxisZColor(double col[4]) const;

    double GetTitleLineWidth() const;
    void GetTitleAColorCoeff(double colCoeff[4]) const;
    void GetTitleInsColorCoeff(double colCoeff[4]) const;

    double GetHelpLineWidth() const;
    void GetHelpInsColor(double col[4]) const;
    void GetHelpBackgroundColor(double col[4]) const;

    double GetParamLineWidth() const;
    void GetParamColor(double col[4]) const;

    double GetParamMeterLineWidth() const;
    void GetParamMeterColor(double col[4]) const;

    double GetParamLinkLineWidth() const;
    void GetParamLinkColor(double col[4]) const;
    
    void GetGrainColor(double col[4]) const;
    void GetGrainColorSelected(double col[4]) const;

    void GetGrainDrawColor(double col[4]) const;
    void GetGrainDrawDefColor(double col[4]) const;

    void GetSoundRectColor(double col[4]) const;
    void GetSoundRectColorSelected(double col[4]) const;
    void GetSoundRectBorderColor(double col[4]) const;
    void GetSoundRectBorderColorSelected(double col[4]) const;
    double GetSoundRectLineWidth() const;

    void GetWaveColor(double col[4]) const;
    double GetWaveLineWidth() const;
    double GetWavePointSize() const;

    void GetSoundRectNameColor(double col[4]) const;
    void GetSoundRectNameColorSelected(double col[4]) const;

    // UIWindow
    void GetUIWindowBackgroundColor(double col[4]) const;
    void GetUIWindowBorderColor(double col[4]) const;
    double GetUIWindowBorderWidth() const;
    void GetUIWindowTitleColor(double col[4]) const;
    double GetUIWindowTitleBoxBorderWidth() const;

    // UIWindow contents
    void GetUIWindowContentsColor(double col[4]) const;
    double GetUIWindowContentsLineWidth() const;
    
protected:
    char _appName[512];
    
    double _backgroundColor[4];

    double _axisLineWidth;
    double _axisXColor[4];
    double _axisYColor[4];
    double _axisZColor[4];

    double _titleLineWidth;
    double _titleAColCoeff[4];
    double _titleInsColCoeff[4];

    double _helpLineWidth;
    double _helpInsColor[4];
    double _helpBackgroundColor[4];

    double _paramLineWidth;
    double _paramColor[4];

    double _grainColor[4];
    double _grainColorSelected[4];

    double _grainVisColor[4];
    double _grainVisDefColor[4];
    
    double _soundRectColor[4];
    double _soundRectColorSelected[4];
    double _soundRectBorderColor[4];
    double _soundRectBorderColorSelected[4];
    double _soundRectLineWidth;

    double _waveColor[4];
    double _waveLineWidth;
    double _wavePointSize;

    double _paramMeterLineWidth;
    double _paramMeterColor[4];
    
    double _paramLinkLineWidth;
    double _paramLinkColor[4];

    double _soundRectNameColor[4];
    double _soundRectNameColorSelected[4];

    // UIWindow
    double _uiWindowBackgroundColor[4];
    double _uiWindowBorderColor[4];
    double _uiWindowBorderWidth;
    double _uiWindowTitleColor[4];
    double _uiWindowTitleBoxBorderWidth;

    // UIWindow contents
    double _uiWindowContentsColor[4];
    double _uiWindowContentsLineWidth;
};

// Default style
class DefaultStyle : public UIStyle
{
public:
    DefaultStyle();
    virtual ~DefaultStyle();
};

// Purple style
class PurpleStyle : public UIStyle
{
public:
    PurpleStyle();
    virtual ~PurpleStyle();
};

// Jellyfish style
class JellyfishStyle : public UIStyle
{
public:
    JellyfishStyle();
    virtual ~JellyfishStyle();
};

// Psytrance style
class PsytranceStyle : public UIStyle
{
public:
    PsytranceStyle();
    virtual ~PsytranceStyle();
};

// Black n white style
class BlackNWhiteStyle : public UIStyle
{
public:
    BlackNWhiteStyle();
    virtual ~BlackNWhiteStyle();
};

// Slug style
class SlugStyle : public UIStyle
{
public:
    SlugStyle();
    virtual ~SlugStyle();
};

#endif
