/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include "UIStyle.h"

#include "UIStyleManager.h"

UIStyleManager::UIStyleManager()
{
    // Default style
    _styleNum = BLACKNWHITE_STYLE;

    _styles[DEFAULT_STYLE] = new DefaultStyle();
    _styles[PURPLE_STYLE] = new PurpleStyle();
    _styles[JELLYFISH_STYLE] = new JellyfishStyle();
    _styles[PSYTRANCE_STYLE] = new PsytranceStyle();
    _styles[BLACKNWHITE_STYLE] = new BlackNWhiteStyle();
    _styles[SLUG_STYLE] = new SlugStyle();
}

UIStyleManager::~UIStyleManager()
{
    for (int i = 0; i < NUM_STYLES; i++)
        delete _styles[i];
}

int
UIStyleManager::GetNumStyles() const
{
    return NUM_STYLES;
}

void
UIStyleManager::SetStyleNum(int styleNum)
{
    if ((styleNum >= 0) && (styleNum < NUM_STYLES))
        _styleNum = styleNum;
}

UIStyle *
UIStyleManager::GetCurrentStyle()
{
    UIStyle *style = _styles[_styleNum];

    return style;
}

int
UIStyleManager::GetCurrentStyleNum() const
{
    return _styleNum;
}
