/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef UI_WINDOW_H
#define UI_WINDOW_H

#include <vector>
using namespace std;

class UIScreen;
class UIStyle;
class UICanvas;
class UIWindow
{
public:
    UIWindow(UIScreen *screen,
             UIStyle *style,
             const char *title,
             double x, double y,
             double width, double height);
    virtual ~UIWindow();

    void SetTitle(const char *title);
    
    void SetVisible(bool flag);
    
    virtual void Draw();
    
    void UpdateStyle(UIStyle *style);

    void GetContentsRect(double *x, double *y,
                         double *width, double *height);
    void AddCanvas(UICanvas *canvas);
    
protected:
    void DrawBackground();
    void DrawBorder();
    void DrawTitle();
    void DrawCanvas();

    vector<UICanvas *> _canvas;
    
    UIScreen *_screen;
    UIStyle *_style;

    bool _visible;
    
    char _title[255];
    double _x;
    double _y;
    
    double _width;
    double _height;
};

#endif
