/* Copyright (C) 2022 Nicolas Dittlo <deadlab.plugins@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef WAVEFORM_PYRAMID_H
#define WAVEFORM_PYRAMID_H

#include <vector>
#include <deque>
using namespace std;

#include "Globals.h"

// Limit the maximum pyramid depth (workaround)
#define DEFAULT_MAX_PYRAMID_LEVEL 8

// Make level of details with samples
class WaveformPyramid
{
public:
    WaveformPyramid(int maxLevel = DEFAULT_MAX_PYRAMID_LEVEL);
    
    virtual ~WaveformPyramid();
    
    void Reset();
    
    void SetValues(const vector<SAMPLE> &samples);

    // Return the number of pushed samples
    long PushValues(const vector<SAMPLE> &samples);
    
    void PopValues(long numSamples);
    
    void ReplaceValues(long start, const vector<SAMPLE> &samples);
    
    void GetValues(SAMPLE start, SAMPLE end, long numValues,
                   vector<SAMPLE> *samples);
    
protected:
    void ResetTmpBuffers();

    void UpsampleResult(int numValues,
                        vector<SAMPLE> *result,
                        const vector<SAMPLE> &vec,
                        SAMPLE leftT, SAMPLE rightT);

    void DecimateSamples(vector<SAMPLE> *result,
                         const vector<SAMPLE> &vec,
                         SAMPLE decFactor);

    int NextPowerOfTwo(int value);

    void VectorToQueue(const vector<SAMPLE> &vec, deque<SAMPLE> *q);
    void QueueToVector(const deque<SAMPLE> &q, vector<SAMPLE> *vec);
    void QueueToVector(const deque<SAMPLE> &q, int queueOffset,
                       vector<SAMPLE> *vec, int numToCopy);

    void PadZerosLeft(vector<SAMPLE> *vec, int size);
    
    void PadZerosRight(vector<SAMPLE> *vec, int size);

    void Interpolate(vector<SAMPLE> *values,
                     bool extendBounds,
                     SAMPLE undefinedValue);
        
    vector<deque<SAMPLE> > _waveformPyramid;
    
    // Keep a push buffer, for pushing by blocks of size power of 2
    deque<SAMPLE> _pushBuf;
    
    long _remainToPop;

    int _maxPyramidLevel;
    
private:
    // Tmp buffers
    vector<vector<SAMPLE> > _tmpBuf0;
    vector<vector<SAMPLE> > _tmpBuf1;
    vector<vector<SAMPLE> > _tmpBuf2;
    vector<vector<SAMPLE> > _tmpBuf3;
    vector<vector<SAMPLE> > _tmpBuf4;
    vector<vector<SAMPLE> > _tmpBuf5;
    vector<SAMPLE> _tmpBuf6;
    vector<vector<SAMPLE> > _tmpBuf7;
    vector<vector<SAMPLE> > _tmpBuf8;
    vector<SAMPLE> _tmpBuf9;
    vector<SAMPLE> _tmpBuf10;
};

#endif
